@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2021 Purism SPC
  *
- * SPDX-License-Identifier: GPL-3-or-later or MIT
+ * SPDX-License-Identifier: GPL-3.0-or-later or MIT
  */
 
 #define G_LOG_DOMAIN "phoc-cursor"
@@ -61,7 +61,7 @@ typedef struct _PhocCursorPrivate {
 
 G_DEFINE_TYPE_WITH_PRIVATE (PhocCursor, phoc_cursor, G_TYPE_OBJECT)
 
-static void handle_pointer_motion (struct wl_listener *listener, void *data);
+static void handle_pointer_motion_relative (struct wl_listener *listener, void *data);
 static void handle_pointer_motion_absolute (struct wl_listener *listener, void *data);
 static void handle_pointer_button (struct wl_listener *listener, void *data);
 static void handle_pointer_axis (struct wl_listener *listener, void *data);
@@ -149,7 +149,7 @@ phoc_cursor_suggest_view_state_change (PhocCursor            *self,
 
   phoc_cursor_view_state_set_view (self, view);
   phoc_cursor_view_state_set_output (self, output);
-  view_get_box (view, &view_box);
+  phoc_view_get_box (view, &view_box);
   priv->view_state.rect = phoc_color_rect_new ((PhocBox *)&view_box,
                                                &PHOC_ANIM_SUGGEST_STATE_CHANGE_COLOR);
   phoc_view_add_bling (view, PHOC_BLING (priv->view_state.rect));
@@ -232,7 +232,7 @@ phoc_cursor_submit_pending_view_state_change (PhocCursor *self)
     phoc_view_maximize (priv->view_state.view, priv->view_state.output);
     break;
   case PHOC_VIEW_STATE_TILED:
-    view_tile (priv->view_state.view, priv->view_state.tile_dir, priv->view_state.output);
+    phoc_view_tile (priv->view_state.view, priv->view_state.tile_dir, priv->view_state.output);
     break;
   case PHOC_VIEW_STATE_FLOATING:
     /* Nothing to do */
@@ -256,9 +256,10 @@ should_ignore_touch_grab (PhocSeat           *seat,
     return false;
 
   struct wlr_surface *root = wlr_surface_get_root_surface (surface);
+  struct wlr_layer_surface_v1 *layer_surface = wlr_layer_surface_v1_try_from_wlr_surface (root);
 
   // FIXME: return false if the grab comes from a xdg-popup that belongs to a layer-surface
-  return wlr_seat_touch_has_grab (seat->seat) && wlr_surface_is_layer_surface (root);
+  return layer_surface && wlr_seat_touch_has_grab (seat->seat);
 }
 
 
@@ -272,9 +273,10 @@ should_ignore_pointer_grab (PhocSeat           *seat,
     return false;
 
   struct wlr_surface *root = wlr_surface_get_root_surface (surface);
+  struct wlr_layer_surface_v1 *layer_surface = wlr_layer_surface_v1_try_from_wlr_surface (root);
 
   // FIXME: return false if the grab comes from a xdg-popup that belongs to a layer-surface
-  return wlr_seat_pointer_has_grab (seat->seat) && wlr_surface_is_layer_surface (root);
+  return layer_surface && wlr_seat_pointer_has_grab (seat->seat);
 }
 
 
@@ -295,7 +297,8 @@ send_pointer_enter (PhocSeat           *seat,
 
 static void
 send_pointer_clear_focus (PhocSeat           *seat,
-                          struct wlr_surface *surface) {
+                          struct wlr_surface *surface)
+{
   if (should_ignore_pointer_grab (seat, surface)) {
     wlr_seat_pointer_clear_focus (seat->seat);
     return;
@@ -310,7 +313,8 @@ send_pointer_motion (PhocSeat           *seat,
                      struct wlr_surface *surface,
                      uint32_t            time,
                      double              sx,
-                     double              sy) {
+                     double              sy)
+{
   if (should_ignore_pointer_grab (seat, surface)) {
     wlr_seat_pointer_send_motion (seat->seat, time, sx, sy);
     return;
@@ -324,7 +328,8 @@ send_pointer_button (PhocSeat             *seat,
                      struct wlr_surface   *surface,
                      uint32_t              time,
                      uint32_t              button,
-                     enum wlr_button_state state) {
+                     enum wlr_button_state state)
+{
   if (should_ignore_pointer_grab (seat, surface)) {
     wlr_seat_pointer_send_button (seat->seat, time, button, state);
     return;
@@ -341,7 +346,8 @@ send_pointer_axis (PhocSeat                 *seat,
                    enum wlr_axis_orientation orientation,
                    double                    value,
                    int32_t                   value_discrete,
-                   enum wlr_axis_source      source) {
+                   enum wlr_axis_source      source)
+{
   if (should_ignore_pointer_grab (seat, surface)) {
     wlr_seat_pointer_send_axis (seat->seat, time, orientation, value, value_discrete, source);
     return;
@@ -451,13 +457,13 @@ phoc_cursor_add_touch_point (PhocCursor *self, struct wlr_touch_down_event *even
   touch_point->lx = lx;
   touch_point->ly = ly;
 
- if (!g_hash_table_insert (priv->touch_points,
-                           GINT_TO_POINTER (event->touch_id),
-                           touch_point)) {
-   g_critical ("Touch point %d already tracked, ignoring", event->touch_id);
- }
+  if (!g_hash_table_insert (priv->touch_points,
+                            GINT_TO_POINTER (event->touch_id),
+                            touch_point)) {
+    g_critical ("Touch point %d already tracked, ignoring", event->touch_id);
+  }
 
- return touch_point;
+  return touch_point;
 }
 
 
@@ -591,7 +597,7 @@ seat_view_deco_motion (PhocSeatView *view, double deco_sx, double deco_sy)
     sy = view->grab_sy;
   }
 
-  PhocViewDecoPart parts = view_get_deco_part (view->view, sx, sy);
+  PhocViewDecoPart parts = phoc_view_get_deco_part (view->view, sx, sy);
 
   bool is_titlebar = (parts & PHOC_VIEW_DECO_PART_TITLEBAR);
   uint32_t edges = 0;
@@ -644,7 +650,7 @@ seat_view_deco_button (PhocSeatView *view, double sx,
     view->has_button_grab = false;
   }
 
-  PhocViewDecoPart parts = view_get_deco_part (view->view, sx, sy);
+  PhocViewDecoPart parts = phoc_view_get_deco_part (view->view, sx, sy);
 
   if (state == WLR_BUTTON_RELEASED && (parts & PHOC_VIEW_DECO_PART_TITLEBAR)) {
     phoc_seat_maybe_set_cursor (view->seat, NULL);
@@ -654,24 +660,25 @@ seat_view_deco_button (PhocSeatView *view, double sx,
 static bool
 phoc_handle_shell_reveal (struct wlr_surface *surface, double lx, double ly, int threshold)
 {
-  PhocServer *server = phoc_server_get_default ();
-  PhocDesktop *desktop = server->desktop;
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
 
   if (surface) {
-    struct wlr_surface *root = wlr_surface_get_root_surface (surface), *iter = root;
+    struct wlr_xdg_surface *wlr_xdg_surface;
+    struct wlr_surface *iter = wlr_surface_get_root_surface (surface);
 
-    while (wlr_surface_is_xdg_surface (iter)) {
-      struct wlr_xdg_surface *xdg_surface = wlr_xdg_surface_from_wlr_surface (iter);
-      if (xdg_surface->role == WLR_XDG_SURFACE_ROLE_POPUP) {
-        iter = xdg_surface->popup->parent;
+    wlr_xdg_surface = wlr_xdg_surface_try_from_wlr_surface (iter);
+    while (wlr_xdg_surface) {
+
+      if (wlr_xdg_surface->role == WLR_XDG_SURFACE_ROLE_POPUP) {
+        iter = wlr_xdg_surface->popup->parent;
+        wlr_xdg_surface = wlr_xdg_surface_try_from_wlr_surface (iter);
       } else {
         break;
       }
     }
 
-    if (wlr_surface_is_layer_surface (iter)) {
+    if (wlr_layer_surface_v1_try_from_wlr_surface (iter))
       return false;
-    }
   }
 
   PhocOutput *output = phoc_desktop_layout_get_output (desktop, lx, ly);
@@ -723,27 +730,23 @@ phoc_handle_shell_reveal (struct wlr_surface *surface, double lx, double ly, int
   return false;
 }
 
+
 static void
-phoc_passthrough_cursor (PhocCursor *self,
-                          uint32_t    time)
+phoc_passthrough_cursor (PhocCursor *self, uint32_t time)
 {
-  PhocServer *server = phoc_server_get_default ();
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
   double sx, sy;
   PhocView *view = NULL;
   PhocSeat *seat = self->seat;
-  PhocDesktop *desktop = server->desktop;
-  struct wlr_surface *surface = phoc_desktop_surface_at (desktop,
-                                                         self->cursor->x, self->cursor->y, &sx, &sy, &view);
-
   struct wl_client *client = NULL;
+  struct wlr_surface *surface;
 
-  if (surface) {
+  surface = phoc_desktop_surface_at (desktop, self->cursor->x, self->cursor->y, &sx, &sy, &view);
+  if (surface)
     client = wl_resource_get_client (surface->resource);
-  }
 
-  if (surface && !phoc_seat_allow_input (seat, surface->resource)) {
+  if (surface && !phoc_seat_allow_input (seat, surface->resource))
     return;
-  }
 
   if (self->cursor_client != client || !client) {
     phoc_seat_maybe_set_cursor (seat, NULL);
@@ -753,16 +756,14 @@ phoc_passthrough_cursor (PhocCursor *self,
   if (view) {
     PhocSeatView *seat_view = phoc_seat_view_from_view (seat, view);
 
-    if (self->pointer_view &&
-        !self->wlr_surface && (surface || seat_view != self->pointer_view)) {
+    if (self->pointer_view && !self->wlr_surface && (surface || seat_view != self->pointer_view))
       seat_view_deco_leave (self->pointer_view);
-    }
 
     self->pointer_view = seat_view;
 
-    if (!surface) {
+    if (!surface)
       seat_view_deco_motion (seat_view, sx, sy);
-    }
+
   } else {
     self->pointer_view = NULL;
   }
@@ -771,16 +772,16 @@ phoc_passthrough_cursor (PhocCursor *self,
 
   if (surface) {
     send_pointer_enter (seat, surface, sx, sy);
-    send_pointer_motion (seat, surface, time, sx ,sy);
+    send_pointer_motion (seat, surface, time, sx,sy);
   } else {
     send_pointer_clear_focus (seat, seat->seat->pointer_state.focused_surface);
   }
 
-  if (seat->drag_icon != NULL) {
+  if (seat->drag_icon != NULL)
     phoc_drag_icon_update_position (seat->drag_icon);
-  }
 }
 
+
 static inline int64_t
 timespec_to_msec (const struct timespec *a)
 {
@@ -799,7 +800,7 @@ phoc_cursor_constructed (GObject *object)
   g_assert (self->xcursor_manager);
 
   wl_signal_add (&wlr_cursor->events.motion, &self->motion);
-  self->motion.notify = handle_pointer_motion;
+  self->motion.notify = handle_pointer_motion_relative;
 
   wl_signal_add (&wlr_cursor->events.motion_absolute, &self->motion_absolute);
   self->motion_absolute.notify = handle_pointer_motion_absolute;
@@ -890,7 +891,7 @@ phoc_cursor_class_init (PhocCursorClass *klass)
 static void
 on_drag_begin (PhocGesture *gesture, double lx, double ly, PhocCursor *self)
 {
-  PhocDesktop *desktop = phoc_server_get_default ()->desktop;
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
   PhocLayerSurface *layer_surface;
   PhocCursorPrivate *priv = phoc_cursor_get_instance_private (self);
   PhocDraggableLayerSurface *drag_surface;
@@ -1025,8 +1026,7 @@ phoc_cursor_update_focus (PhocCursor *self)
 void
 phoc_cursor_update_position (PhocCursor *self, uint32_t time)
 {
-  PhocServer *server = phoc_server_get_default ();
-  PhocDesktop *desktop = server->desktop;
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
   PhocSeat *seat = self->seat;
   PhocView *view;
 
@@ -1047,7 +1047,7 @@ phoc_cursor_update_position (PhocCursor *self, uint32_t time)
 
       bool output_is_landscape = output_box.width > output_box.height;
 
-      if (view_is_fullscreen (view)) {
+      if (phoc_view_is_fullscreen (view)) {
         phoc_view_set_fullscreen (view, true, output);
       } else if (self->cursor->y < output_box.y + PHOC_EDGE_SNAP_THRESHOLD) {
         phoc_cursor_suggest_view_state_change (self, view, output, PHOC_VIEW_STATE_MAXIMIZED, -1);
@@ -1067,7 +1067,7 @@ phoc_cursor_update_position (PhocCursor *self, uint32_t time)
                                                PHOC_VIEW_TILE_RIGHT);
       } else {
         phoc_cursor_clear_view_state_change (self);
-        view_restore (view);
+        phoc_view_restore (view);
         phoc_view_move (view, self->view_x + dx - geom.x * phoc_view_get_scale (view),
                         self->view_y + dy - geom.y * phoc_view_get_scale (view));
       }
@@ -1102,9 +1102,7 @@ phoc_cursor_update_position (PhocCursor *self, uint32_t time)
       } else if (self->resize_edges & WLR_EDGE_RIGHT) {
         width += dx;
       }
-      phoc_view_move_resize (view, x, y,
-                             width < 1 ? 1 : width,
-                             height < 1 ? 1 : height);
+      phoc_view_move_resize (view, x, y, MAX (1, width), MAX (1, height));
     }
     break;
   default:
@@ -1113,54 +1111,53 @@ phoc_cursor_update_position (PhocCursor *self, uint32_t time)
 }
 
 static void
-phoc_cursor_press_button (PhocCursor *self,
-                          struct wlr_input_device *device, uint32_t time, uint32_t button,
-                          uint32_t state, double lx, double ly)
+phoc_cursor_press_button (PhocCursor              *self,
+                          struct wlr_input_device *device,
+                          uint32_t                 time,
+                          uint32_t                 button,
+                          uint32_t                 state,
+                          double                   lx,
+                          double                   ly)
 {
-  PhocServer *server = phoc_server_get_default ();
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
   PhocCursorPrivate *priv = phoc_cursor_get_instance_private (self);
   PhocSeat *seat = self->seat;
-  PhocDesktop *desktop = server->desktop;
-
   bool is_touch = device->type == WLR_INPUT_DEVICE_TOUCH;
-
   double sx, sy;
   PhocView *view;
-  struct wlr_surface *surface = phoc_desktop_surface_at (desktop,
-                                                         lx, ly, &sx, &sy, &view);
+  struct wlr_surface *surface;
 
-  if (state == WLR_BUTTON_PRESSED && view &&
-      phoc_seat_has_meta_pressed (seat)) {
+  surface = phoc_desktop_surface_at (desktop, lx, ly, &sx, &sy, &view);
+  if (state == WLR_BUTTON_PRESSED && view && phoc_seat_grab_meta_press (seat)) {
     phoc_seat_set_focus_view (seat, view);
 
-    uint32_t edges;
     switch (button) {
     case BTN_LEFT:
       phoc_seat_begin_move (seat, view);
       break;
-    case BTN_RIGHT:
-      edges = 0;
-      if (sx < view->wlr_surface->current.width/2) {
+    case BTN_RIGHT: {
+      uint32_t edges = 0;
+
+      if (sx < view->wlr_surface->current.width/2)
         edges |= WLR_EDGE_LEFT;
-      } else {
+      else
         edges |= WLR_EDGE_RIGHT;
-      }
-      if (sy < view->wlr_surface->current.height/2) {
+
+      if (sy < view->wlr_surface->current.height/2)
         edges |= WLR_EDGE_TOP;
-      } else {
+      else
         edges |= WLR_EDGE_BOTTOM;
-      }
+
       phoc_seat_begin_resize (seat, view, edges);
       break;
+    }
     default:
       /* don't care */
       break;
     }
   } else {
-    if (view && !surface && self->pointer_view) {
-      seat_view_deco_button (self->pointer_view,
-                             sx, sy, button, state);
-    }
+    if (view && !surface && self->pointer_view)
+      seat_view_deco_button (self->pointer_view, sx, sy, button, state);
 
     if (state == WLR_BUTTON_RELEASED && self->mode != PHOC_CURSOR_PASSTHROUGH) {
       if (priv->view_state.view)
@@ -1170,32 +1167,29 @@ phoc_cursor_press_button (PhocCursor *self,
     }
 
     if (state == WLR_BUTTON_PRESSED) {
-      if (view) {
+      if (view)
         phoc_seat_set_focus_view (seat, view);
-      }
-      if (surface && wlr_surface_is_layer_surface (surface)) {
-        struct wlr_layer_surface_v1 *layer =
-          wlr_layer_surface_v1_from_wlr_surface (surface);
-        if (layer->current.keyboard_interactive) {
+
+      if (surface) {
+        struct wlr_layer_surface_v1 *layer = wlr_layer_surface_v1_try_from_wlr_surface (surface);
+        if (layer && layer->current.keyboard_interactive)
           phoc_seat_set_focus_layer (seat, layer);
-        }
       }
     }
   }
 
-  if (!phoc_handle_shell_reveal (surface, lx, ly, PHOC_SHELL_REVEAL_POINTER_THRESHOLD) && !is_touch) {
+  if (!phoc_handle_shell_reveal (surface, lx, ly, PHOC_SHELL_REVEAL_POINTER_THRESHOLD) && !is_touch)
     send_pointer_button (seat, surface, time, button, state);
-  }
 }
 
 
 static void
-handle_pointer_motion (struct wl_listener *listener, void *data)
+handle_pointer_motion_relative (struct wl_listener *listener, void *data)
 {
+  PhocServer *server = phoc_server_get_default ();
+  PhocDesktop *desktop = phoc_server_get_desktop (server);
   PhocCursor *self = wl_container_of (listener, self, motion);
   struct wlr_pointer_motion_event *event = data;
-  PhocServer *server = phoc_server_get_default ();
-  PhocDesktop *desktop = server->desktop;
   double dx = event->delta_x;
   double dy = event->delta_y;
 
@@ -1205,7 +1199,7 @@ handle_pointer_motion (struct wl_listener *listener, void *data)
   phoc_desktop_notify_activity (desktop, self->seat);
 
   wlr_relative_pointer_manager_v1_send_relative_motion (
-    server->desktop->relative_pointer_manager,
+    desktop->relative_pointer_manager,
     self->seat->seat, (uint64_t)event->time_msec * 1000, dx, dy,
     dx_unaccel, dy_unaccel);
 
@@ -1243,10 +1237,10 @@ handle_pointer_motion (struct wl_listener *listener, void *data)
 static void
 handle_pointer_motion_absolute (struct wl_listener *listener, void *data)
 {
+  PhocServer *server = phoc_server_get_default ();
+  PhocDesktop *desktop = phoc_server_get_desktop (server);
   PhocCursor *self = wl_container_of (listener, self, motion_absolute);
   struct wlr_pointer_motion_absolute_event *event = data;
-  PhocServer *server = phoc_server_get_default ();
-  PhocDesktop *desktop = server->desktop;
   double lx, ly;
 
   phoc_desktop_notify_activity (desktop, self->seat);
@@ -1259,7 +1253,7 @@ handle_pointer_motion_absolute (struct wl_listener *listener, void *data)
   handle_gestures_for_event_at (self, lx, ly, PHOC_EVENT_MOTION_NOTIFY, event, sizeof (*event));
 
   wlr_relative_pointer_manager_v1_send_relative_motion (
-    server->desktop->relative_pointer_manager,
+    desktop->relative_pointer_manager,
     self->seat->seat, (uint64_t)event->time_msec * 1000, dx, dy, dx, dy);
 
   if (self->pointer_view) {
@@ -1279,10 +1273,9 @@ handle_pointer_motion_absolute (struct wl_listener *listener, void *data)
 static void
 handle_pointer_button (struct wl_listener *listener, void *data)
 {
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
   PhocCursor *self = wl_container_of (listener, self, button);
   struct wlr_pointer_button_event *event = data;
-  PhocServer *server = phoc_server_get_default ();
-  PhocDesktop *desktop = server->desktop;
   PhocEventType type;
   bool is_touch = event->pointer->base.type == WLR_INPUT_DEVICE_TOUCH;
 
@@ -1320,10 +1313,9 @@ phoc_cursor_handle_event (PhocCursor   *self,
 static void
 handle_pointer_axis (struct wl_listener *listener, void *data)
 {
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
   PhocCursor *self = wl_container_of (listener, self, axis);
   struct wlr_pointer_axis_event *event = data;
-  PhocServer *server = phoc_server_get_default ();
-  PhocDesktop *desktop = server->desktop;
 
   phoc_desktop_notify_activity (desktop, self->seat);
   send_pointer_axis (self->seat, self->seat->seat->pointer_state.focused_surface, event->time_msec,
@@ -1333,9 +1325,8 @@ handle_pointer_axis (struct wl_listener *listener, void *data)
 static void
 handle_pointer_frame (struct wl_listener *listener, void *data)
 {
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
   PhocCursor *self = wl_container_of (listener, self, frame);
-  PhocServer *server = phoc_server_get_default ();
-  PhocDesktop *desktop = server->desktop;
 
   phoc_desktop_notify_activity (desktop, self->seat);
   wlr_seat_pointer_notify_frame (self->seat->seat);
@@ -1351,7 +1342,7 @@ phoc_cursor_handle_touch_down (PhocCursor                  *self,
                                struct wlr_touch_down_event *event)
 {
   PhocServer *server = phoc_server_get_default ();
-  PhocDesktop *desktop = server->desktop;
+  PhocDesktop *desktop = phoc_server_get_desktop (server);
   PhocSeat *seat = self->seat;
   PhocTouchPoint *touch_point;
   double lx, ly;
@@ -1380,17 +1371,16 @@ phoc_cursor_handle_touch_down (PhocCursor                  *self,
     if (view)
       phoc_seat_set_focus_view (seat, view);
 
-    if (wlr_surface_is_layer_surface (root)) {
-      struct wlr_layer_surface_v1 *wlr_layer = wlr_layer_surface_v1_from_wlr_surface (root);
-
+    struct wlr_layer_surface_v1 *wlr_layer = wlr_layer_surface_v1_try_from_wlr_surface (root);
+    if (wlr_layer) {
       /* TODO: Use press gesture */
       if (wlr_layer->current.keyboard_interactive) {
-          phoc_seat_set_focus_layer (seat, wlr_layer);
+        phoc_seat_set_focus_layer (seat, wlr_layer);
       }
     }
   }
 
-  if (G_UNLIKELY (server->debug_flags & PHOC_SERVER_DEBUG_FLAG_TOUCH_POINTS)) {
+  if (G_UNLIKELY (phoc_server_check_debug_flags (server, PHOC_SERVER_DEBUG_FLAG_TOUCH_POINTS))) {
     PhocOutput *output;
     wl_list_for_each (output, &desktop->outputs, link) {
       if (wlr_output_layout_contains_point (desktop->layout, output->wlr_output, lx, ly)) {
@@ -1402,7 +1392,7 @@ phoc_cursor_handle_touch_down (PhocCursor                  *self,
           .width = 1,
           .height = 1
         };
-        wlr_output_damage_add_box (output->damage, &box);
+        wlr_damage_ring_add_box (&output->damage_ring, &box);
       }
     }
   }
@@ -1454,8 +1444,7 @@ void
 phoc_cursor_handle_touch_motion (PhocCursor                    *self,
                                  struct wlr_touch_motion_event *event)
 {
-  PhocServer *server = phoc_server_get_default ();
-  PhocDesktop *desktop = server->desktop;
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
   struct wlr_touch_point *point;
   PhocTouchPoint *touch_point;
   double lx, ly;
@@ -1484,8 +1473,9 @@ phoc_cursor_handle_touch_motion (PhocCursor                    *self,
     float scale = 1.0;
 
     struct wlr_surface *root = wlr_surface_get_root_surface (surface);
-    if (wlr_surface_is_layer_surface (root)) {
-      struct wlr_layer_surface_v1 *wlr_layer_surface = wlr_layer_surface_v1_from_wlr_surface (root);
+    struct wlr_layer_surface_v1 *wlr_layer_surface = wlr_layer_surface_v1_try_from_wlr_surface (root);
+    if (wlr_layer_surface) {
+
       struct wlr_box output_box;
       wlr_output_layout_get_box (desktop->layout, output->wlr_output, &output_box);
 
@@ -1530,8 +1520,11 @@ phoc_cursor_handle_touch_motion (PhocCursor                    *self,
 
     if (found) {
       struct wlr_surface *sub = surface;
-      while (sub && wlr_surface_is_subsurface (sub)) {
-        struct wlr_subsurface *subsurface = wlr_subsurface_from_wlr_surface (sub);
+      while (sub) {
+        struct wlr_subsurface *subsurface = wlr_subsurface_try_from_wlr_surface (sub);
+        if (subsurface == NULL)
+          break;
+
         sx -= subsurface->current.x;
         sy -= subsurface->current.y;
         sub = subsurface->parent;
@@ -1550,6 +1543,9 @@ phoc_cursor_handle_touch_motion (PhocCursor                    *self,
       wlr_cursor_warp (self->cursor, NULL, lx, ly);
       phoc_cursor_update_position (self, event->time_msec);
     }
+
+    if (self->seat->drag_icon != NULL)
+      phoc_drag_icon_update_position (self->seat->drag_icon);
   }
 }
 
@@ -1640,7 +1636,7 @@ void
 phoc_cursor_handle_focus_change (PhocCursor                                 *self,
                                  struct wlr_seat_pointer_focus_change_event *event)
 {
-  PhocServer *server = phoc_server_get_default ();
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
   double sx = event->sx;
   double sy = event->sy;
 
@@ -1652,7 +1648,7 @@ phoc_cursor_handle_focus_change (PhocCursor                                 *sel
 
   phoc_cursor_constrain (self,
                          wlr_pointer_constraints_v1_constraint_for_surface (
-                           server->desktop->pointer_constraints,
+                           desktop->pointer_constraints,
                            event->new_surface, self->seat->seat),
                          sx, sy);
 }
@@ -1660,8 +1656,7 @@ phoc_cursor_handle_focus_change (PhocCursor                                 *sel
 static void
 phoc_cursor_handle_constraint_commit (PhocCursor *self)
 {
-  PhocServer *server = phoc_server_get_default ();
-  PhocDesktop *desktop = server->desktop;
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
 
   double sx, sy;
   struct wlr_surface *surface = phoc_desktop_surface_at (desktop,
@@ -1747,9 +1742,9 @@ phoc_cursor_constrain (PhocCursor *self,
 PhocCursor *
 phoc_cursor_new (PhocSeat *seat)
 {
-  return PHOC_CURSOR (g_object_new (PHOC_TYPE_CURSOR,
-                                    "seat", seat,
-                                    NULL));
+  return g_object_new (PHOC_TYPE_CURSOR,
+                       "seat", seat,
+                       NULL);
 }
 
 
