@@ -12,8 +12,8 @@
 #include <wlr/types/wlr_cursor.h>
 #include <wlr/types/wlr_data_control_v1.h>
 #include <wlr/types/wlr_export_dmabuf_v1.h>
+#include <wlr/types/wlr_fractional_scale_v1.h>
 #include <wlr/types/wlr_gamma_control_v1.h>
-#include <wlr/types/wlr_idle.h>
 #include <wlr/types/wlr_idle_notify_v1.h>
 #include <wlr/types/wlr_input_inhibitor.h>
 #include <wlr/types/wlr_layer_shell_v1.h>
@@ -21,6 +21,7 @@
 #include <wlr/types/wlr_output_power_management_v1.h>
 #include <wlr/types/wlr_pointer_constraints_v1.h>
 #include <wlr/types/wlr_primary_selection_v1.h>
+#include <wlr/types/wlr_screencopy_v1.h>
 #include <wlr/types/wlr_server_decoration.h>
 #include <wlr/types/wlr_single_pixel_buffer_v1.h>
 #include <wlr/types/wlr_tablet_v2.h>
@@ -33,7 +34,6 @@
 #include <wlr/types/wlr_xdg_output_v1.h>
 #include <wlr/types/wlr_xdg_shell.h>
 #include <wlr/util/box.h>
-#include <wlr/version.h>
 #include "cursor.h"
 #include "device-state.h"
 #include "idle-inhibit.h"
@@ -51,8 +51,14 @@
 #include "layer-shell-effects.h"
 
 #include "xdg-surface.h"
+#include "xdg-toplevel-decoration.h"
 #include "xwayland-surface.h"
 
+/* Maximum protocol versions we support */
+#define PHOC_FRACTIONAL_SCALE_VERSION 1
+#define PHOC_XDG_SHELL_VERSION 5
+#define PHOC_LAYER_SHELL_VERSION 3
+
 /**
  * PhocDesktop:
  *
@@ -61,7 +67,6 @@
 
 enum {
   PROP_0,
-  PROP_CONFIG,
   PROP_SCALE_TO_FIT,
   PROP_LAST_PROP,
 };
@@ -69,6 +74,8 @@ static GParamSpec *props[PROP_LAST_PROP];
 
 
 typedef struct _PhocDesktopPrivate {
+  GQueue                *views;
+
   PhocIdleInhibit       *idle_inhibit;
 
   gboolean               enable_animations;
@@ -77,7 +84,10 @@ typedef struct _PhocDesktopPrivate {
   GSettings             *interface_settings;
 
   /* Protocols from wlroots */
+  struct wlr_data_control_manager_v1 *data_control_manager_v1;
   struct wlr_idle_notifier_v1 *idle_notifier_v1;
+  struct wlr_screencopy_manager_v1 *screencopy_manager_v1;
+  struct wl_listener gamma_control_set_gamma;
 
   /* Protocols without upstreamable implementations */
   PhocPhoshPrivate      *phosh;
@@ -100,10 +110,6 @@ phoc_desktop_set_property (GObject      *object,
   PhocDesktop *self = PHOC_DESKTOP (object);
 
   switch (property_id) {
-  case PROP_CONFIG:
-    self->config = g_value_get_pointer (value);
-    g_object_notify_by_pspec (G_OBJECT (self), props[PROP_CONFIG]);
-    break;
   case PROP_SCALE_TO_FIT:
     phoc_desktop_set_scale_to_fit (self, g_value_get_boolean (value));
     break;
@@ -123,9 +129,6 @@ phoc_desktop_get_property (GObject    *object,
   PhocDesktop *self = PHOC_DESKTOP (object);
 
   switch (property_id) {
-  case PROP_CONFIG:
-    g_value_set_pointer (value, self->config);
-    break;
   case PROP_SCALE_TO_FIT:
     g_value_set_boolean (value, phoc_desktop_get_scale_to_fit (self));
     break;
@@ -158,7 +161,7 @@ view_at (PhocView *view, double lx, double ly, struct wlr_surface **surface, dou
     return true;
   }
 
-  if (view_get_deco_part (view, view_sx, view_sy)) {
+  if (phoc_view_get_deco_part (view, view_sx, view_sy)) {
     if (sx)
       *sx = view_sx;
     if (sy)
@@ -171,17 +174,17 @@ view_at (PhocView *view, double lx, double ly, struct wlr_surface **surface, dou
 }
 
 static PhocView *
-desktop_view_at (PhocDesktop         *desktop,
+desktop_view_at (PhocDesktop         *self,
                  double               lx,
                  double               ly,
                  struct wlr_surface **surface,
                  double              *sx,
                  double              *sy)
 {
-  PhocView *view;
+  for (GList *l = phoc_desktop_get_views (self)->head; l; l = l->next) {
+    PhocView *view = PHOC_VIEW (l->data);
 
-  wl_list_for_each (view, &desktop->views, link) {
-    if (phoc_desktop_view_is_visible (desktop, view) && view_at (view, lx, ly, surface, sx, sy))
+    if (phoc_desktop_view_is_visible (self, view) && view_at (view, lx, ly, surface, sx, sy))
       return view;
   }
   return NULL;
@@ -323,25 +326,32 @@ phoc_desktop_surface_at(PhocDesktop *desktop,
 }
 
 gboolean
-phoc_desktop_view_is_visible (PhocDesktop *desktop, PhocView *view)
+phoc_desktop_view_is_visible (PhocDesktop *self, PhocView *view)
 {
+  PhocDesktopPrivate *priv;
+  PhocView *top_view;
+
+  g_assert (PHOC_IS_DESKTOP (self));
+  g_assert (PHOC_IS_VIEW (view));
+
+  priv = phoc_desktop_get_instance_private (self);
+
   if (!phoc_view_is_mapped (view)) {
     return false;
   }
 
-  g_assert_false (wl_list_empty (&desktop->views));
+  g_assert_true (priv->views->head);
 
-  if (wl_list_length (&desktop->outputs) != 1) {
+  if (wl_list_length (&self->outputs) != 1) {
     // current heuristics work well only for single output
     return true;
   }
 
-  if (!desktop->maximize) {
+  if (!self->maximize) {
     return true;
   }
 
-  PhocView *top_view = wl_container_of (desktop->views.next, view, link);
-
+  top_view = phoc_desktop_get_view_by_index (self, 0);
 #ifdef PHOC_XWAYLAND
   // XWayland parent relations can be complicated and aren't described by PhocView
   // relationships very well at the moment, so just make all XWayland windows visible
@@ -356,7 +366,7 @@ phoc_desktop_view_is_visible (PhocDesktop *desktop, PhocView *view)
     if (v == view) {
       return true;
     }
-    if (view_is_maximized (v)) {
+    if (phoc_view_is_maximized (v)) {
       return false;
     }
     v = v->parent;
@@ -369,10 +379,10 @@ static void
 handle_layout_change (struct wl_listener *listener, void *data)
 {
   PhocDesktop *self;
+  PhocDesktopPrivate *priv;
   struct wlr_output *center_output;
   struct wlr_box center_output_box;
   double center_x, center_y;
-  PhocView *view;
   PhocOutput *output;
 
   self = wl_container_of (listener, self, layout_change);
@@ -380,17 +390,20 @@ handle_layout_change (struct wl_listener *listener, void *data)
   if (center_output == NULL)
     return;
 
+  priv = phoc_desktop_get_instance_private (self);
   wlr_output_layout_get_box (self->layout, center_output, &center_output_box);
   center_x = center_output_box.x + center_output_box.width / 2;
   center_y = center_output_box.y + center_output_box.height / 2;
 
   /* Make sure all views are on an existing output */
-  wl_list_for_each (view, &self->views, link) {
+  for (GList *l = priv->views->head; l; l = l->next) {
+    PhocView *view = PHOC_VIEW (l->data);
     struct wlr_box box;
-    view_get_box (view, &box);
 
+    phoc_view_get_box (view, &box);
     if (wlr_output_layout_intersects (self->layout, NULL, &box))
       continue;
+
     phoc_view_move (view, center_x - box.width / 2, center_y - box.height / 2);
   }
 
@@ -404,9 +417,9 @@ static void
 input_inhibit_activate (struct wl_listener *listener, void *data)
 {
   PhocDesktop *desktop = wl_container_of(listener, desktop, input_inhibit_activate);
-  PhocServer *server = phoc_server_get_default ();
+  PhocInput *input = phoc_server_get_input (phoc_server_get_default ());
 
-  for (GSList *elem = phoc_input_get_seats (server->input); elem; elem = elem->next) {
+  for (GSList *elem = phoc_input_get_seats (input); elem; elem = elem->next) {
     PhocSeat *seat = PHOC_SEAT (elem->data);
 
     g_assert (PHOC_IS_SEAT (seat));
@@ -418,9 +431,9 @@ input_inhibit_activate (struct wl_listener *listener, void *data)
 static void
 input_inhibit_deactivate (struct wl_listener *listener, void *data)
 {
-  PhocServer *server = phoc_server_get_default ();
+  PhocInput *input = phoc_server_get_input (phoc_server_get_default ());
 
-  for (GSList *elem = phoc_input_get_seats (server->input); elem; elem = elem->next) {
+  for (GSList *elem = phoc_input_get_seats (input); elem; elem = elem->next) {
     PhocSeat *seat = PHOC_SEAT (elem->data);
 
     g_assert (PHOC_IS_SEAT (seat));
@@ -458,24 +471,21 @@ handle_constraint_destroy (struct wl_listener *listener, void *data)
   free(constraint);
 }
 
-static
-void handle_pointer_constraint (struct wl_listener *listener, void *data)
+static void
+handle_pointer_constraint (struct wl_listener *listener, void *data)
 {
-  PhocServer *server = phoc_server_get_default ();
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
   struct wlr_pointer_constraint_v1 *wlr_constraint = data;
   PhocSeat *seat = wlr_constraint->seat->data;
   PhocCursor *cursor = phoc_seat_get_cursor (seat);
 
-  PhocPointerConstraint *constraint =
-    calloc(1, sizeof(PhocPointerConstraint));
-  constraint->constraint = wlr_constraint;
-
+  PhocPointerConstraint *constraint = g_new0 (PhocPointerConstraint, 1);
   constraint->destroy.notify = handle_constraint_destroy;
   wl_signal_add(&wlr_constraint->events.destroy, &constraint->destroy);
 
   double sx, sy;
   struct wlr_surface *surface = phoc_desktop_surface_at(
-    server->desktop,
+    desktop,
     cursor->cursor->x, cursor->cursor->y, &sx, &sy, NULL);
 
   if (surface == wlr_constraint->surface) {
@@ -524,6 +534,7 @@ static void
 handle_xwayland_ready (struct wl_listener *listener,
                        void               *data)
 {
+  PhocInput *input = phoc_server_get_input (phoc_server_get_default ());
   PhocDesktop *desktop = wl_container_of (listener, desktop, xwayland_ready);
   xcb_connection_t *xcb_conn = xcb_connect (NULL, NULL);
 
@@ -544,7 +555,7 @@ handle_xwayland_ready (struct wl_listener *listener,
 
     if (error) {
       g_warning ("could not resolve atom %s, X11 error code %d",
-        atom_map[i], error->error_code);
+                 atom_map[i], error->error_code);
       free (error);
     }
 
@@ -558,8 +569,7 @@ handle_xwayland_ready (struct wl_listener *listener,
 
 #ifdef PHOC_XWAYLAND
   if (desktop->xwayland != NULL) {
-    PhocSeat *xwayland_seat = phoc_input_get_seat (phoc_server_get_default ()->input,
-                                                   PHOC_CONFIG_DEFAULT_SEAT_NAME);
+    PhocSeat *xwayland_seat = phoc_input_get_seat (input, PHOC_CONFIG_DEFAULT_SEAT_NAME);
     wlr_xwayland_set_seat (desktop->xwayland, xwayland_seat->seat);
   }
 #endif
@@ -585,8 +595,6 @@ handle_xwayland_remove_startup_id (struct wl_listener *listener, void *data)
 static void
 handle_xwayland_surface (struct wl_listener *listener, void *data)
 {
-  PhocDesktop *desktop = wl_container_of (listener, desktop, xwayland_surface);
-
   struct wlr_xwayland_surface *surface = data;
   g_debug ("new xwayland surface: title=%s, class=%s, instance=%s",
            surface->title, surface->class, surface->instance);
@@ -644,45 +652,45 @@ phoc_desktop_setup_xwayland (PhocDesktop *self)
 {
 #ifdef PHOC_XWAYLAND
   const char *cursor_default = PHOC_XCURSOR_DEFAULT;
-  PhocConfig *config = self->config;
   PhocServer *server = phoc_server_get_default ();
+  PhocConfig *config = phoc_server_get_config (server);
 
-  self->xcursor_manager = wlr_xcursor_manager_create(NULL, PHOC_XCURSOR_SIZE);
+  self->xcursor_manager = wlr_xcursor_manager_create (NULL, PHOC_XCURSOR_SIZE);
   g_return_if_fail (self->xcursor_manager);
 
   if (config->xwayland) {
-    self->xwayland = wlr_xwayland_create(server->wl_display,
-                                         server->compositor, config->xwayland_lazy);
+    struct wl_display *wl_display = phoc_server_get_wl_display (server);
+    struct wlr_compositor *wlr_compositor = phoc_server_get_compositor (server);
+
+    self->xwayland = wlr_xwayland_create (wl_display, wlr_compositor, config->xwayland_lazy);
     if (!self->xwayland) {
       g_critical ("Failed to initialize Xwayland");
       g_unsetenv ("DISPLAY");
       return;
     }
 
-    wl_signal_add(&self->xwayland->events.new_surface,
-                  &self->xwayland_surface);
+    wl_signal_add (&self->xwayland->events.new_surface, &self->xwayland_surface);
     self->xwayland_surface.notify = handle_xwayland_surface;
 
-    wl_signal_add(&self->xwayland->events.ready,
-                  &self->xwayland_ready);
+    wl_signal_add (&self->xwayland->events.ready, &self->xwayland_ready);
     self->xwayland_ready.notify = handle_xwayland_ready;
 
-    wl_signal_add(&self->xwayland->events.remove_startup_info,
-                  &self->xwayland_remove_startup_id);
+    wl_signal_add (&self->xwayland->events.remove_startup_info, &self->xwayland_remove_startup_id);
     self->xwayland_remove_startup_id.notify = handle_xwayland_remove_startup_id;
 
     g_setenv ("DISPLAY", self->xwayland->display_name, true);
 
-    if (!wlr_xcursor_manager_load(self->xcursor_manager, 1))
+    if (!wlr_xcursor_manager_load (self->xcursor_manager, 1))
       g_critical ("Cannot load XWayland XCursor theme");
 
-    struct wlr_xcursor *xcursor = wlr_xcursor_manager_get_xcursor(
-                                                                  self->xcursor_manager, cursor_default, 1);
+    struct wlr_xcursor *xcursor = wlr_xcursor_manager_get_xcursor (self->xcursor_manager,
+                                                                   cursor_default,
+                                                                   1);
     if (xcursor != NULL) {
       struct wlr_xcursor_image *image = xcursor->images[0];
-      wlr_xwayland_set_cursor(self->xwayland, image->buffer,
-                              image->width * 4, image->width, image->height, image->hotspot_x,
-                              image->hotspot_y);
+      wlr_xwayland_set_cursor (self->xwayland, image->buffer,
+                               image->width * 4, image->width, image->height, image->hotspot_x,
+                               image->hotspot_y);
     }
   }
 #endif
@@ -695,30 +703,31 @@ phoc_desktop_constructed (GObject *object)
   PhocDesktop *self = PHOC_DESKTOP (object);
   PhocDesktopPrivate *priv = phoc_desktop_get_instance_private (self);
   PhocServer *server = phoc_server_get_default ();
+  struct wl_display *wl_display = phoc_server_get_wl_display (server);
+  struct wlr_backend *wlr_backend = phoc_server_get_backend (server);
 
   G_OBJECT_CLASS (phoc_desktop_parent_class)->constructed (object);
 
-  wl_list_init (&self->views);
   wl_list_init (&self->outputs);
 
   self->new_output.notify = handle_new_output;
-  wl_signal_add (&server->backend->events.new_output, &self->new_output);
+  wl_signal_add (&wlr_backend->events.new_output, &self->new_output);
 
   self->layout = wlr_output_layout_create ();
-  wlr_xdg_output_manager_v1_create (server->wl_display, self->layout);
+  wlr_xdg_output_manager_v1_create (wl_display, self->layout);
   self->layout_change.notify = handle_layout_change;
   wl_signal_add (&self->layout->events.change, &self->layout_change);
 
-  self->xdg_shell = wlr_xdg_shell_create (server->wl_display, 5);
-  wl_signal_add (&self->xdg_shell->events.new_surface, &self->xdg_shell_surface);
-  self->xdg_shell_surface.notify = handle_xdg_shell_surface;
+  self->xdg_shell = wlr_xdg_shell_create(wl_display, PHOC_XDG_SHELL_VERSION);
+  wl_signal_add(&self->xdg_shell->events.new_surface, &self->xdg_shell_surface);
+  self->xdg_shell_surface.notify = phoc_handle_xdg_shell_surface;
 
-  self->layer_shell = wlr_layer_shell_v1_create (server->wl_display);
-  wl_signal_add (&self->layer_shell->events.new_surface, &self->layer_shell_surface);
-  self->layer_shell_surface.notify = handle_layer_shell_surface;
+  self->layer_shell = wlr_layer_shell_v1_create (wl_display, PHOC_LAYER_SHELL_VERSION);
+  wl_signal_add(&self->layer_shell->events.new_surface, &self->layer_shell_surface);
+  self->layer_shell_surface.notify = phoc_handle_layer_shell_surface;
   priv->layer_shell_effects = phoc_layer_shell_effects_new ();
 
-  self->tablet_v2 = wlr_tablet_v2_create (server->wl_display);
+  self->tablet_v2 = wlr_tablet_v2_create (wl_display);
 
   char cursor_size_fmt[16];
   snprintf (cursor_size_fmt, sizeof (cursor_size_fmt), "%d", PHOC_XCURSOR_SIZE);
@@ -726,80 +735,84 @@ phoc_desktop_constructed (GObject *object)
 
   phoc_desktop_setup_xwayland (self);
 
-  self->gamma_control_manager_v1 = wlr_gamma_control_manager_v1_create (server->wl_display);
-  self->export_dmabuf_manager_v1 = wlr_export_dmabuf_manager_v1_create (server->wl_display);
-  self->server_decoration_manager = wlr_server_decoration_manager_create (server->wl_display);
+  self->security_context_manager_v1 = wlr_security_context_manager_v1_create (wl_display);
+
+  self->gamma_control_manager_v1 = wlr_gamma_control_manager_v1_create (wl_display);
+  priv->gamma_control_set_gamma.notify = phoc_output_handle_gamma_control_set_gamma;
+  wl_signal_add (&self->gamma_control_manager_v1->events.set_gamma, &priv->gamma_control_set_gamma);
+
+  self->export_dmabuf_manager_v1 = wlr_export_dmabuf_manager_v1_create (wl_display);
+  self->server_decoration_manager = wlr_server_decoration_manager_create (wl_display);
   wlr_server_decoration_manager_set_default_mode (self->server_decoration_manager,
                                                   WLR_SERVER_DECORATION_MANAGER_MODE_CLIENT);
   self->primary_selection_device_manager =
-    wlr_primary_selection_v1_device_manager_create (server->wl_display);
+    wlr_primary_selection_v1_device_manager_create (wl_display);
 
-  self->input_inhibit = wlr_input_inhibit_manager_create (server->wl_display);
+  self->input_inhibit = wlr_input_inhibit_manager_create (wl_display);
   self->input_inhibit_activate.notify = input_inhibit_activate;
   wl_signal_add (&self->input_inhibit->events.activate, &self->input_inhibit_activate);
   self->input_inhibit_deactivate.notify = input_inhibit_deactivate;
   wl_signal_add (&self->input_inhibit->events.deactivate, &self->input_inhibit_deactivate);
 
-  self->input_method = wlr_input_method_manager_v2_create (server->wl_display);
-  self->text_input = wlr_text_input_manager_v3_create (server->wl_display);
+  self->input_method = wlr_input_method_manager_v2_create (wl_display);
+  self->text_input = wlr_text_input_manager_v3_create (wl_display);
 
-  self->idle = wlr_idle_create (server->wl_display);
-  priv->idle_notifier_v1 = wlr_idle_notifier_v1_create (server->wl_display);
-  priv->idle_inhibit = phoc_idle_inhibit_create (self->idle);
+  priv->idle_notifier_v1 = wlr_idle_notifier_v1_create (wl_display);
+  priv->idle_inhibit = phoc_idle_inhibit_create ();
 
-  priv->gtk_shell = phoc_gtk_shell_create (self, server->wl_display);
+  priv->gtk_shell = phoc_gtk_shell_create (self, wl_display);
   priv->phosh = phoc_phosh_private_new ();
 
-  self->xdg_activation_v1 = wlr_xdg_activation_v1_create (server->wl_display);
+  self->xdg_activation_v1 = wlr_xdg_activation_v1_create (wl_display);
   self->xdg_activation_v1_request_activate.notify = phoc_xdg_activation_v1_handle_request_activate;
   wl_signal_add (&self->xdg_activation_v1->events.request_activate,
                  &self->xdg_activation_v1_request_activate);
 
-  self->virtual_keyboard = wlr_virtual_keyboard_manager_v1_create (server->wl_display);
+  self->virtual_keyboard = wlr_virtual_keyboard_manager_v1_create (wl_display);
   wl_signal_add (&self->virtual_keyboard->events.new_virtual_keyboard,
                  &self->virtual_keyboard_new);
   self->virtual_keyboard_new.notify = phoc_handle_virtual_keyboard;
 
-  self->virtual_pointer = wlr_virtual_pointer_manager_v1_create (server->wl_display);
+  self->virtual_pointer = wlr_virtual_pointer_manager_v1_create (wl_display);
   wl_signal_add (&self->virtual_pointer->events.new_virtual_pointer, &self->virtual_pointer_new);
   self->virtual_pointer_new.notify = phoc_handle_virtual_pointer;
 
-  self->screencopy = wlr_screencopy_manager_v1_create (server->wl_display);
+  priv->screencopy_manager_v1 = wlr_screencopy_manager_v1_create (wl_display);
 
-  self->xdg_decoration_manager = wlr_xdg_decoration_manager_v1_create (server->wl_display);
+  self->xdg_decoration_manager = wlr_xdg_decoration_manager_v1_create (wl_display);
   wl_signal_add (&self->xdg_decoration_manager->events.new_toplevel_decoration,
                  &self->xdg_toplevel_decoration);
 
-  self->xdg_toplevel_decoration.notify = handle_xdg_toplevel_decoration;
-  wlr_viewporter_create (server->wl_display);
-  wlr_single_pixel_buffer_manager_v1_create (server->wl_display);
+  self->xdg_toplevel_decoration.notify = phoc_handle_xdg_toplevel_decoration;
+  wlr_viewporter_create (wl_display);
+  wlr_single_pixel_buffer_manager_v1_create (wl_display);
+  wlr_fractional_scale_manager_v1_create (wl_display, PHOC_FRACTIONAL_SCALE_VERSION);
 
-  struct wlr_xdg_foreign_registry *foreign_registry =
-                wlr_xdg_foreign_registry_create (server->wl_display);
-  wlr_xdg_foreign_v1_create (server->wl_display, foreign_registry);
-  wlr_xdg_foreign_v2_create (server->wl_display, foreign_registry);
+  struct wlr_xdg_foreign_registry *foreign_registry = wlr_xdg_foreign_registry_create (wl_display);
+  wlr_xdg_foreign_v1_create (wl_display, foreign_registry);
+  wlr_xdg_foreign_v2_create (wl_display, foreign_registry);
 
-  self->pointer_constraints = wlr_pointer_constraints_v1_create (server->wl_display);
+  self->pointer_constraints = wlr_pointer_constraints_v1_create (wl_display);
   self->pointer_constraint.notify = handle_pointer_constraint;
   wl_signal_add (&self->pointer_constraints->events.new_constraint, &self->pointer_constraint);
 
-  self->presentation = wlr_presentation_create (server->wl_display, server->backend);
-  self->foreign_toplevel_manager_v1 = wlr_foreign_toplevel_manager_v1_create (server->wl_display);
-  self->relative_pointer_manager = wlr_relative_pointer_manager_v1_create (server->wl_display);
-  self->pointer_gestures = wlr_pointer_gestures_v1_create (server->wl_display);
+  self->presentation = wlr_presentation_create (wl_display, wlr_backend);
+  self->foreign_toplevel_manager_v1 = wlr_foreign_toplevel_manager_v1_create (wl_display);
+  self->relative_pointer_manager = wlr_relative_pointer_manager_v1_create (wl_display);
+  self->pointer_gestures = wlr_pointer_gestures_v1_create (wl_display);
 
-  self->output_manager_v1 = wlr_output_manager_v1_create (server->wl_display);
-  self->output_manager_apply.notify = handle_output_manager_apply;
+  self->output_manager_v1 = wlr_output_manager_v1_create (wl_display);
+  self->output_manager_apply.notify = phoc_handle_output_manager_apply;
   wl_signal_add (&self->output_manager_v1->events.apply, &self->output_manager_apply);
-  self->output_manager_test.notify = handle_output_manager_test;
+  self->output_manager_test.notify = phoc_handle_output_manager_test;
   wl_signal_add (&self->output_manager_v1->events.test, &self->output_manager_test);
 
-  self->output_power_manager_v1 = wlr_output_power_manager_v1_create (server->wl_display);
+  self->output_power_manager_v1 = wlr_output_power_manager_v1_create (wl_display);
   self->output_power_manager_set_mode.notify = phoc_output_handle_output_power_manager_set_mode;
   wl_signal_add (&self->output_power_manager_v1->events.set_mode,
                  &self->output_power_manager_set_mode);
 
-  wlr_data_control_manager_v1_create (server->wl_display);
+  priv->data_control_manager_v1 = wlr_data_control_manager_v1_create (wl_display);
 
   /* sm.puri.phosh settings */
   priv->settings = g_settings_new ("sm.puri.phoc");
@@ -810,7 +823,7 @@ phoc_desktop_constructed (GObject *object)
 
   /* org.gnome.desktop.interface settings */
   priv->interface_settings = g_settings_new ("org.gnome.desktop.interface");
-  if (server->debug_flags & PHOC_SERVER_DEBUG_FLAG_DISABLE_ANIMATIONS) {
+  if (phoc_server_check_debug_flags (server, PHOC_SERVER_DEBUG_FLAG_DISABLE_ANIMATIONS)) {
     priv->enable_animations = FALSE;
   } else {
     g_signal_connect_swapped (priv->interface_settings, "changed::enable-animations",
@@ -826,6 +839,8 @@ phoc_desktop_finalize (GObject *object)
   PhocDesktop *self = PHOC_DESKTOP (object);
   PhocDesktopPrivate *priv = phoc_desktop_get_instance_private (self);
 
+  g_clear_pointer (&priv->views, g_queue_free);
+
   /* TODO: currently destroys the backend before the desktop */
   //wl_list_remove (&self->new_output.link);
   wl_list_remove (&self->layout_change.link);
@@ -884,13 +899,6 @@ phoc_desktop_class_init (PhocDesktopClass *klass)
   object_class->constructed = phoc_desktop_constructed;
   object_class->finalize = phoc_desktop_finalize;
 
-  props[PROP_CONFIG] =
-    g_param_spec_pointer (
-      "config",
-      "Config",
-      "The config object",
-      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-
   /**
    * PhocDesktop:scale-to-fit:
    *
@@ -911,6 +919,7 @@ phoc_desktop_init (PhocDesktop *self)
   PhocDesktopPrivate *priv;
 
   priv = phoc_desktop_get_instance_private (self);
+  priv->views = g_queue_new ();
   priv->enable_animations = TRUE;
 
   self->input_output_map = g_hash_table_new_full (g_str_hash,
@@ -921,48 +930,12 @@ phoc_desktop_init (PhocDesktop *self)
 
 
 PhocDesktop *
-phoc_desktop_new (PhocConfig *config)
+phoc_desktop_new (void)
 {
-  return g_object_new (PHOC_TYPE_DESKTOP, "config", config, NULL);
+  return g_object_new (PHOC_TYPE_DESKTOP, NULL);
 }
 
 
-/**
- * phoc_desktop_toggle_output_blank:
- *
- * Blank or unblank all outputs depending on the current state.
- * Brings all outputs into consistent state (either all blanked
- * or all enabled).
- */
-void
-phoc_desktop_toggle_output_blank (PhocDesktop *self)
-{
-  PhocOutput *output;
-
-  gboolean enable = true;
-  wl_list_for_each(output, &self->outputs, link) {
-    if (output->wlr_output->enabled) {
-      enable = false;
-      break;
-    }
-  }
-
-  wl_list_for_each(output, &self->outputs, link) {
-    if (!wlr_output_layout_get (self->layout, output->wlr_output))
-      continue;
-    bool prior = output->wlr_output->enabled;
-    wlr_output_enable (output->wlr_output, enable);
-    if (!wlr_output_commit (output->wlr_output)) {
-      /* Reset the enabled state if committing failed. */
-      g_warning ("Could not set output enabled state to %d", enable);
-      wlr_output_enable (output->wlr_output, prior);
-      continue;
-    }
-    if (enable)
-      phoc_output_damage_whole(output);
-  }
-}
-
 /**
  * phoc_desktop_set_auto_maximize:
  *
@@ -971,10 +944,10 @@ phoc_desktop_toggle_output_blank (PhocDesktop *self)
 void
 phoc_desktop_set_auto_maximize (PhocDesktop *self, gboolean enable)
 {
-  PhocView *view;
+  PhocDesktopPrivate *priv = phoc_desktop_get_instance_private (self);
   PhocServer *server = phoc_server_get_default();
 
-  if (G_UNLIKELY (server->debug_flags & PHOC_SERVER_DEBUG_FLAG_AUTO_MAXIMIZE)) {
+  if (G_UNLIKELY (phoc_server_check_debug_flags (server, PHOC_SERVER_DEBUG_FLAG_AUTO_MAXIMIZE))) {
     if (enable == FALSE)
       g_info ("Not disabling auto-maximize due to `auto-maximize` debug flag");
     enable = TRUE;
@@ -985,15 +958,20 @@ phoc_desktop_set_auto_maximize (PhocDesktop *self, gboolean enable)
 
   /* Disabling auto-maximize leaves all views in their current position */
   if (!enable) {
-    PhocInput *input = phoc_server_get_default()->input;
+    PhocInput *input = phoc_server_get_input (server);
+
+    for (GList *l = priv->views->head; l; l = l->next) {
+      PhocView *view = PHOC_VIEW (l->data);
 
-    wl_list_for_each (view, &self->views, link)
       phoc_view_appear_activated (view, phoc_input_view_has_focus (input, view));
+    }
     return;
   }
 
-  wl_list_for_each (view, &self->views, link) {
-    view_auto_maximize (view);
+  for (GList *l = priv->views->head; l; l = l->next) {
+    PhocView *view = PHOC_VIEW (l->data);
+
+    phoc_view_auto_maximize (view);
     phoc_view_appear_activated (view, true);
   }
 }
@@ -1026,7 +1004,7 @@ phoc_desktop_set_scale_to_fit (PhocDesktop *self, gboolean enable)
 gboolean
 phoc_desktop_get_scale_to_fit (PhocDesktop *self)
 {
-    return self->scale_to_fit;
+  return self->scale_to_fit;
 }
 
 /**
@@ -1072,12 +1050,41 @@ phoc_desktop_find_output (PhocDesktop *self,
 
   wl_list_for_each (output, &self->outputs, link) {
     if (phoc_output_is_match (output, make, model, serial))
-        return output;
+      return output;
   }
 
   return NULL;
 }
 
+/**
+ * phoc_desktop_find_output_by_name:
+ * @self: The desktop
+ * @name: The output's name
+
+ * Find an output by it's name.
+ *
+ * Returns: (transfer none) (nullable): The matching output or
+ *  %NULL if no output matches.
+ */
+PhocOutput *
+phoc_desktop_find_output_by_name (PhocDesktop *self, const char *name)
+{
+  PhocOutput *output;
+
+  g_assert (PHOC_IS_DESKTOP (self));
+
+  if (!name)
+    return NULL;
+
+  wl_list_for_each (output, &self->outputs, link) {
+    if (g_strcmp0 (phoc_output_get_name (output), name) == 0)
+      return output;
+  }
+
+  return NULL;
+}
+
+
 /**
  * phoc_desktop_layer_surface_at:
  * @self: The `PhocDesktop` to look the surface up for
@@ -1091,8 +1098,8 @@ phoc_desktop_find_output (PhocDesktop *self,
  *
  * Returns: (transfer none) (nullable): The `PhocLayerSurface`
  */
-PhocLayerSurface
-*phoc_desktop_layer_surface_at(PhocDesktop *self, double lx, double ly, double *sx, double *sy)
+PhocLayerSurface *
+phoc_desktop_layer_surface_at (PhocDesktop *self, double lx, double ly, double *sx, double *sy)
 {
   struct wlr_surface *wlr_surface;
   struct wlr_layer_surface_v1 *wlr_layer_surface;
@@ -1106,12 +1113,11 @@ PhocLayerSurface
   if (!wlr_surface)
     return NULL;
 
-  if (!wlr_surface_is_layer_surface (wlr_surface))
+  wlr_layer_surface = wlr_layer_surface_v1_try_from_wlr_surface (wlr_surface);
+  if (wlr_layer_surface == NULL)
     return NULL;
 
-  wlr_layer_surface = wlr_layer_surface_v1_from_wlr_surface (wlr_surface);
-  layer_surface = wlr_layer_surface->data;
-
+  layer_surface = PHOC_LAYER_SURFACE (wlr_layer_surface->data);
   if (sx)
     *sx = sx_;
 
@@ -1140,7 +1146,7 @@ phoc_desktop_get_builtin_output (PhocDesktop *self)
 
   wl_list_for_each (output, &self->outputs, link) {
     if (phoc_output_is_builtin (output))
-        return output;
+      return output;
   }
 
   return NULL;
@@ -1241,6 +1247,200 @@ phoc_desktop_notify_activity (PhocDesktop *self, PhocSeat *seat)
   g_assert (PHOC_IS_DESKTOP (self));
   priv = phoc_desktop_get_instance_private (self);
 
-  wlr_idle_notify_activity (self->idle, seat->seat);
   wlr_idle_notifier_v1_notify_activity (priv->idle_notifier_v1, seat->seat);
 }
+
+gboolean
+phoc_desktop_is_privileged_protocol (PhocDesktop *self, const struct wl_global *global)
+{
+  gboolean is_priv;
+  PhocDesktopPrivate *priv;
+
+  g_return_val_if_fail (PHOC_IS_DESKTOP (self), TRUE);
+  priv = phoc_desktop_get_instance_private (self);
+
+  is_priv = (
+    global == phoc_phosh_private_get_global (priv->phosh) ||
+    global == phoc_layer_shell_effects_get_global (priv->layer_shell_effects) ||
+    global == priv->data_control_manager_v1->global ||
+    global == priv->screencopy_manager_v1->global ||
+    global == self->export_dmabuf_manager_v1->global ||
+    global == self->foreign_toplevel_manager_v1->global ||
+    global == self->gamma_control_manager_v1->global ||
+    global == self->input_inhibit->global ||
+    global == self->input_method->global ||
+    global == self->layer_shell->global ||
+    global == self->output_manager_v1->global ||
+    global == self->output_power_manager_v1->global ||
+    global == self->security_context_manager_v1->global ||
+    global == self->virtual_keyboard->global ||
+    global == self->virtual_pointer->global
+    );
+
+  return is_priv;
+}
+
+/**
+ * phoc_desktop_get_views:
+ * @self: the desktop
+ *
+ * Get the current views. Don't manipulate the queue directly. This is
+ * only meant for reading.
+ *
+ * Returns:(transfer none): The views.
+ */
+GQueue *
+phoc_desktop_get_views (PhocDesktop *self)
+{
+  PhocDesktopPrivate *priv;
+
+  g_assert (PHOC_IS_DESKTOP (self));
+  priv = phoc_desktop_get_instance_private (self);
+
+  return priv->views;
+}
+
+/**
+ * phoc_desktop_move_view_to_top:
+ * @self: the desktop
+ * @view: a view
+ *
+ * Move the given view to the front of the view stack meaning that it
+ * will be rendered on top of other views (but below move-to-top
+ * views if `view` isn't a `move-to-top-view` itself).
+ */
+void
+phoc_desktop_move_view_to_top (PhocDesktop *self, PhocView *view)
+{
+  GList *view_link;
+  PhocDesktopPrivate *priv;
+
+  g_assert (PHOC_IS_DESKTOP (self));
+  priv = phoc_desktop_get_instance_private (self);
+
+  view_link = g_queue_find (priv->views, view);
+  g_assert (view_link);
+
+  g_queue_unlink (priv->views, view_link);
+
+  if (G_UNLIKELY (phoc_view_is_always_on_top (view))) {
+    g_queue_push_head_link (priv->views, view_link);
+  } else {
+    GList *l = NULL;
+
+    for (l = phoc_desktop_get_views (self)->head; l; l = l->next) {
+      if (!phoc_view_is_always_on_top (PHOC_VIEW (l->data)))
+        break;
+    }
+
+    g_queue_insert_before_link (priv->views, l, view_link);
+  }
+
+  phoc_view_damage_whole (view);
+}
+
+/**
+ * phoc_desktop_has_views:
+ * @self: the desktop
+ *
+ * Check whether the desktop has any views.
+ *
+ * Returns: %TRUE if there's at least on view, otherwise %FALSE
+ */
+gboolean
+phoc_desktop_has_views (PhocDesktop *self)
+{
+  PhocDesktopPrivate *priv;
+
+  g_assert (PHOC_IS_DESKTOP (self));
+  priv = phoc_desktop_get_instance_private (self);
+
+  return !!priv->views->head;
+}
+
+/**
+ * phoc_desktop_get_view_by_index:
+ * @self: the desktop
+ * @index: the index to get the view for
+ *
+ * Gets the view at the given position in the queue. If the view is
+ * not part of that desktop %NULL is returned.
+ *
+ * Returns:(transfer none)(nullable): the looked up view
+ */
+PhocView *
+phoc_desktop_get_view_by_index (PhocDesktop *self, guint index)
+{
+  PhocDesktopPrivate *priv;
+
+  g_assert (PHOC_IS_DESKTOP (self));
+  priv = phoc_desktop_get_instance_private (self);
+
+  return g_queue_peek_nth (priv->views, index);
+}
+
+/**
+ * phoc_desktop_insert_view:
+ * @self: the desktop
+ * @view: the view to insert
+ *
+ * Insert the view into the queue of views. New views are inserted
+ * at the front so they appear on top of other views.
+ */
+void
+phoc_desktop_insert_view (PhocDesktop *self, PhocView *view)
+{
+  PhocDesktopPrivate *priv;
+
+  g_assert (PHOC_IS_DESKTOP (self));
+  priv = phoc_desktop_get_instance_private (self);
+
+  g_queue_push_head (priv->views, view);
+  phoc_desktop_move_view_to_top (self, view);
+}
+
+/**
+ * phoc_desktop_remove_view:
+ * @self: the desktop
+ * @view: The view to remove
+ *
+ * Removes a view from the queue of views.
+ *
+ * Returns: %TRUE if the view was found, otherwise %FALSE
+ */
+gboolean
+phoc_desktop_remove_view (PhocDesktop *self, PhocView *view)
+{
+  PhocDesktopPrivate *priv;
+
+  g_assert (PHOC_IS_DESKTOP (self));
+  priv = phoc_desktop_get_instance_private (self);
+
+  return g_queue_remove (priv->views, view);
+}
+
+/**
+ * phoc_desktop_set_view_always_on_top:
+ * @self: The desktop singleton
+ * @view: The view to operate on
+ * @on_top: Whether to render this view on top of other views
+ *
+ * If `on_top` is `TRUE` marks the `view` (and it's children) as being
+ * placed on top of other views. If `on_top` if `FALSE` the `view` will
+ * be placed normally.
+ */
+void
+phoc_desktop_set_view_always_on_top (PhocDesktop *self, PhocView *view, gboolean on_top)
+{
+  PhocView *child;
+
+  g_assert (PHOC_IS_DESKTOP (self));
+  g_assert (PHOC_IS_VIEW (view));
+
+  phoc_view_set_always_on_top (view, on_top);
+  phoc_desktop_move_view_to_top (self, view);
+
+  /* Raise children recursively */
+  wl_list_for_each_reverse (child, &view->stack, parent_link)
+    phoc_desktop_set_view_always_on_top (self, child, on_top);
+}
