@@ -43,11 +43,13 @@ struct _PhocKeyboard {
   GSettings *input_settings;
   GSettings *keyboard_settings;
   struct xkb_keymap *keymap;
-  uint32_t meta_key;
   GnomeXkbInfo *xkbinfo;
 
   xkb_keysym_t pressed_keysyms_translated[PHOC_KEYBOARD_PRESSED_KEYSYMS_CAP];
   xkb_keysym_t pressed_keysyms_raw[PHOC_KEYBOARD_PRESSED_KEYSYMS_CAP];
+
+  uint32_t meta_key;
+  bool     meta_press_valid;
 };
 G_DEFINE_TYPE(PhocKeyboard, phoc_keyboard, PHOC_TYPE_INPUT_DEVICE)
 
@@ -56,7 +58,7 @@ enum {
   ACTIVITY,
   N_SIGNALS
 };
-static guint signals [N_SIGNALS];
+static guint signals[N_SIGNALS];
 
 
 static ssize_t
@@ -71,8 +73,11 @@ pressed_keysyms_index(const xkb_keysym_t *pressed_keysyms,
   return -1;
 }
 
-static size_t pressed_keysyms_length(const xkb_keysym_t *pressed_keysyms) {
+static size_t
+pressed_keysyms_length (const xkb_keysym_t *pressed_keysyms)
+{
   size_t n = 0;
+
   for (size_t i = 0; i < PHOC_KEYBOARD_PRESSED_KEYSYMS_CAP; ++i) {
     if (pressed_keysyms[i] != XKB_KEY_NoSymbol) {
       ++n;
@@ -155,10 +160,10 @@ keyboard_execute_compositor_binding(PhocKeyboard *self,
   if (keysym >= XKB_KEY_XF86Switch_VT_1 &&
       keysym <= XKB_KEY_XF86Switch_VT_12) {
 
-    struct wlr_session *session = wlr_backend_get_session(server->backend);
+    struct wlr_session *session = phoc_server_get_session (server);
     if (session) {
       unsigned vt = keysym - XKB_KEY_XF86Switch_VT_1 + 1;
-      wlr_session_change_vt(session, vt);
+      wlr_session_change_vt (session, vt);
     }
 
     return true;
@@ -192,7 +197,6 @@ keyboard_execute_super_key (PhocKeyboard              *self,
                             uint32_t                   time,
                             enum wl_keyboard_key_state state)
 {
-  static bool press_valid = false;
   uint32_t super_mod = 1 << xkb_map_mod_get_index(self->keymap, "Mod4");
 
   if (modifiers > 0 && modifiers != super_mod) {
@@ -206,11 +210,11 @@ keyboard_execute_super_key (PhocKeyboard              *self,
 
       if (state == WL_KEYBOARD_KEY_STATE_PRESSED) {
         /* The super key was pressed wait for a release */
-        press_valid = true;
+        self->meta_press_valid = true;
         return phoc_phosh_private_forward_keysym (&combo, time, false);
-      } else if (press_valid == true) {
+      } else if (self->meta_press_valid == true) {
         /* The super key was released and is valid (no other key was pressed) */
-        press_valid = false;
+        self->meta_press_valid = false;
         return phoc_phosh_private_forward_keysym (&combo, time, true);
       }
 
@@ -219,29 +223,7 @@ keyboard_execute_super_key (PhocKeyboard              *self,
       /* A non super key was pressed, don't wait for the release
        * as the super key is a modifier.
        */
-      press_valid = false;
-    }
-  }
-
-  return false;
-}
-
-static bool
-keyboard_execute_power_key (PhocKeyboard              *self,
-                            const xkb_keysym_t        *keysyms,
-                            size_t                     keysyms_len,
-                            enum wl_keyboard_key_state state)
-{
-  PhocServer *server = phoc_server_get_default ();
-
-  if (state != WL_KEYBOARD_KEY_STATE_PRESSED)
-      return false;
-
-  for (size_t i = 0; i < keysyms_len; ++i) {
-    if (keysyms[i] == XKB_KEY_XF86PowerDown || keysyms[i] == XKB_KEY_XF86PowerOff) {
-      g_debug ("Power button pressed");
-      phoc_desktop_toggle_output_blank (server->desktop);
-      return true;
+      self->meta_press_valid = false;
     }
   }
 
@@ -263,8 +245,7 @@ keyboard_execute_binding (PhocKeyboard              *self,
                           size_t                     keysyms_len,
                           enum wl_keyboard_key_state state)
 {
-  PhocServer *server = phoc_server_get_default ();
-  PhocKeybindings *keybindings;
+  PhocConfig *config = phoc_server_get_config (phoc_server_get_default ());
   PhocSeat *seat = phoc_input_device_get_seat (PHOC_INPUT_DEVICE (self));
 
   if (state != WL_KEYBOARD_KEY_STATE_PRESSED)
@@ -277,10 +258,8 @@ keyboard_execute_binding (PhocKeyboard              *self,
     }
   }
 
-  size_t n = pressed_keysyms_length(pressed_keysyms);
-  keybindings = server->config->keybindings;
-
-  if (phoc_keybindings_handle_pressed (keybindings, modifiers, pressed_keysyms, n, seat))
+  size_t n = pressed_keysyms_length (pressed_keysyms);
+  if (phoc_keybindings_handle_pressed (config->keybindings, modifiers, pressed_keysyms, n, seat))
     return true;
 
   return false;
@@ -428,11 +407,6 @@ phoc_keyboard_handle_key(PhocKeyboard *self, struct wlr_keyboard_key_event *even
                                                    event->state);
   }
 
-  // Check for the power button after the susbscribed bindings so clients can override it
-  if (!handled) {
-    handled = keyboard_execute_power_key (self, keysyms, keysyms_len, event->state);
-  }
-
   if (!handled) {
     PhocInputDevice *input_device = PHOC_INPUT_DEVICE (self);
     struct wlr_input_device *device = phoc_input_device_get_device (input_device);
@@ -484,7 +458,7 @@ set_fallback_keymap (PhocKeyboard *self)
 
   context = xkb_context_new (XKB_CONTEXT_NO_FLAGS);
   if (context == NULL) {
-       return;
+    return;
   }
 
   xkb_keymap_unref (self->keymap);
@@ -798,17 +772,30 @@ phoc_keyboard_next_layout (PhocKeyboard *self)
   g_settings_set_value(self->input_settings, "sources", g_variant_builder_end(&builder));
 }
 
-
 /**
- * phoc_keyboard_get_meta_key:
+ * phoc_keyboard_grab_meta_press:
  * @self: the keyboard
  *
- * Returns: the current Meta key
+ * If the meta key is currently the only pressed modifier grab it
+ * (hence canceling the meta key press/release sequence) making it
+ * available to other gestures (like left-click + meta).
+ *
+ * Returns: %TRUE if the meta key is pressed. Otherwise %FALSE.
  */
-uint32_t
-phoc_keyboard_get_meta_key (PhocKeyboard *self)
+gboolean
+phoc_keyboard_grab_meta_press (PhocKeyboard *self)
 {
+  uint32_t modifiers;
+  struct wlr_input_device *device;
+
   g_assert (PHOC_IS_KEYBOARD (self));
 
-  return self->meta_key;
+  device = phoc_input_device_get_device (PHOC_INPUT_DEVICE (self));
+  modifiers = wlr_keyboard_get_modifiers (wlr_keyboard_from_input_device (device));
+  if ((modifiers ^ self->meta_key) == 0) {
+    self->meta_press_valid = false;
+    return TRUE;
+  }
+
+  return FALSE;
 }
