@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2022 Purism SPC
  *
- * SPDX-License-Identifier: GPL-3.0+
+ * SPDX-License-Identifier: GPL-3.0-or-later
  * Author: Guido Günther <agx@sigxcpu.org>
  */
 
@@ -199,8 +199,8 @@ handle_draggable_layer_surface_set_threshold (struct wl_client   *client,
   if (drag_surface->layer_surface == NULL)
     return;
 
-  threshold = (threshold < 1.0) ? threshold : 1.0;
-  threshold = (threshold > 0.0) ? threshold : 0.0;
+  threshold = MIN (threshold, 1.0);
+  threshold = MAX (threshold, 0.0);
 
   drag_surface->pending.threshold = threshold;
 }
@@ -462,8 +462,8 @@ surface_handle_commit (struct wl_listener *listener, void *data)
 
   /* Update animation end in case it's ongoing to compensate for size changes */
   if (changed && drag_surface->drag.anim_dir == ANIM_DIR_IN) {
-      drag_surface->drag.anim_end = drag_surface->current.folded;
-      phoc_draggable_layer_surface_slide (drag_surface, ANIM_DIR_IN);
+    drag_surface->drag.anim_end = drag_surface->current.folded;
+    phoc_draggable_layer_surface_slide (drag_surface, ANIM_DIR_IN);
   }
 
   /* TODO: cancel related gestures on drag mode changes */
@@ -498,10 +498,10 @@ alpha_surface_handle_commit (struct wl_listener *listener, void *data)
 
   phoc_layer_surface_get_output (layer_surface);
   output = phoc_layer_surface_get_output (layer_surface);
-  phoc_output_damage_whole_local_surface (output,
-                                          layer_surface->layer_surface->surface,
-                                          layer_surface->geo.x, layer_surface->geo.y);
-
+  phoc_output_damage_whole_surface (output,
+                                    layer_surface->layer_surface->surface,
+                                    layer_surface->geo.x,
+                                    layer_surface->geo.y);
 }
 
 
@@ -697,12 +697,12 @@ phoc_layer_shell_effects_class_init (PhocLayerShellEffectsClass *klass)
 static void
 phoc_layer_shell_effects_init (PhocLayerShellEffects *self)
 {
-  struct wl_display *display = phoc_server_get_default ()->wl_display;
+  struct wl_display *wl_display = phoc_server_get_wl_display (phoc_server_get_default ());
 
   self->drag_surfaces_by_layer_surface = g_hash_table_new (g_direct_hash, g_direct_equal);
   self->alpha_surfaces_by_layer_surface = g_hash_table_new (g_direct_hash, g_direct_equal);
 
-  self->global = wl_global_create (display, &zphoc_layer_shell_effects_v1_interface,
+  self->global = wl_global_create (wl_display, &zphoc_layer_shell_effects_v1_interface,
                                    LAYER_SHELL_EFFECTS_VERSION, self, layer_shell_effects_bind);
 
 }
@@ -711,7 +711,16 @@ phoc_layer_shell_effects_init (PhocLayerShellEffects *self)
 PhocLayerShellEffects *
 phoc_layer_shell_effects_new (void)
 {
-  return PHOC_LAYER_SHELL_EFFECTS (g_object_new (PHOC_TYPE_LAYER_SHELL_EFFECTS, NULL));
+  return g_object_new (PHOC_TYPE_LAYER_SHELL_EFFECTS, NULL);
+}
+
+
+struct wl_global *
+phoc_layer_shell_effects_get_global (PhocLayerShellEffects *self)
+{
+  g_assert (PHOC_IS_LAYER_SHELL_EFFECTS (self));
+
+  return self->global;
 }
 
 
@@ -753,8 +762,15 @@ apply_margin (PhocDraggableLayerSurface *drag_surface, double margin)
 static void
 apply_state (PhocDraggableLayerSurface *drag_surface, PhocDraggableSurfaceState state)
 {
+  struct wlr_output *wlr_output;
+
   drag_surface->state = state;
-  phoc_output_update_shell_reveal (PHOC_OUTPUT (drag_surface->layer_surface->layer_surface->output->data));
+
+  wlr_output = drag_surface->layer_surface->layer_surface->output;
+  if (wlr_output) {
+    PhocOutput *output = PHOC_OUTPUT (wlr_output->data);
+    phoc_output_update_shell_reveal (output);
+  }
 }
 
 
@@ -895,7 +911,7 @@ phoc_draggable_layer_surface_slide (PhocDraggableLayerSurface *drag_surface, Pho
   apply_state (drag_surface, PHOC_DRAGGABLE_SURFACE_STATE_ANIMATING);
 
   g_debug ("%s: start: %d, end: %d dir: %d", __func__,
-          drag_surface->drag.anim_start, drag_surface->drag.anim_end, drag_surface->drag.anim_dir);
+           drag_surface->drag.anim_start, drag_surface->drag.anim_end, drag_surface->drag.anim_dir);
 
 
   if (drag_surface->drag.anim_id) {
@@ -1002,10 +1018,10 @@ accept_drag (PhocDraggableLayerSurface *drag_surface,
 PhocDraggableSurfaceState
 phoc_draggable_layer_surface_drag_start (PhocDraggableLayerSurface *drag_surface, double lx, double ly)
 {
-  PhocServer *server = phoc_server_get_default ();
+  PhocDesktop *desktop = phoc_server_get_desktop (phoc_server_get_default ());
   struct wlr_layer_surface_v1 *wlr_layer_surface = drag_surface->layer_surface->layer_surface;
   struct wlr_box output_box;
-  wlr_output_layout_get_box (server->desktop->layout, wlr_layer_surface->output, &output_box);
+  wlr_output_layout_get_box (desktop->layout, wlr_layer_surface->output, &output_box);
   double sx = lx - drag_surface->geo.x - output_box.x;
   double sy = ly - drag_surface->geo.y - output_box.y;
   bool is_handle = false;
