@@ -19,6 +19,7 @@
 #include "output.h"
 #include "seat.h"
 #include "server.h"
+#include "utils.h"
 
 #include <glib.h>
 
@@ -30,9 +31,9 @@ apply_exclusive (struct wlr_box *usable_area,
                  int32_t margin_top, int32_t margin_right,
                  int32_t margin_bottom, int32_t margin_left)
 {
-  if (exclusive <= 0) {
+  if (exclusive <= 0)
     return;
-  }
+
   struct {
     uint32_t anchors;
     int *positive_axis;
@@ -76,14 +77,13 @@ apply_exclusive (struct wlr_box *usable_area,
       .margin = margin_right,
     },
   };
-  for (size_t i = 0; i < sizeof(edges) / sizeof(edges[0]); ++i) {
+  for (size_t i = 0; i < sizeof (edges) / sizeof (edges[0]); ++i) {
     if ((anchor & edges[i].anchors) == edges[i].anchors && exclusive + edges[i].margin > 0) {
-      if (edges[i].positive_axis) {
+      if (edges[i].positive_axis)
         *edges[i].positive_axis += exclusive + edges[i].margin;
-      }
-      if (edges[i].negative_axis) {
+
+      if (edges[i].negative_axis)
         *edges[i].negative_axis -= exclusive + edges[i].margin;
-      }
     }
   }
 }
@@ -92,26 +92,27 @@ static void
 update_cursors (PhocLayerSurface *layer_surface, GSList *seats /* PhocSeat */)
 {
   PhocServer *server = phoc_server_get_default ();
+  PhocDesktop *desktop = phoc_server_get_desktop (server);
+  PhocInput *input = phoc_server_get_input (server);
 
-  for (GSList *elem = phoc_input_get_seats (server->input); elem; elem = elem->next) {
+  for (GSList *elem = phoc_input_get_seats (input); elem; elem = elem->next) {
     PhocSeat *seat = PHOC_SEAT (elem->data);
 
     g_assert (PHOC_IS_SEAT (seat));
-    PhocCursor *cursor = phoc_seat_get_cursor(seat);
+    PhocCursor *cursor = phoc_seat_get_cursor (seat);
     double sx, sy;
 
-    struct wlr_surface *surface = phoc_desktop_surface_at(
-      server->desktop,
+    struct wlr_surface *surface = phoc_desktop_surface_at (
+      desktop,
       cursor->cursor->x, cursor->cursor->y, &sx, &sy, NULL);
 
     if (surface == layer_surface->layer_surface->surface) {
       struct timespec time;
-      if (clock_gettime(CLOCK_MONOTONIC, &time) == 0) {
-        phoc_cursor_update_position(cursor,
-                                    time.tv_sec * 1000 + time.tv_nsec / 1000000);
+      if (clock_gettime (CLOCK_MONOTONIC, &time) == 0) {
+        phoc_cursor_update_position (cursor,
+                                     time.tv_sec * 1000 + time.tv_nsec / 1000000);
       } else {
-        g_critical ("Failed to get time, not updating position. Errno: %s\n",
-                    strerror(errno));
+        g_critical ("Failed to get time, not updating position. Errno: %s\n", strerror (errno));
       }
     }
   }
@@ -203,7 +204,7 @@ arrange_layer (PhocOutput                     *output,
     // Apply
     struct wlr_box old_geo = layer_surface->geo;
     layer_surface->geo = box;
-    if (wlr_layer_surface->mapped) {
+    if (wlr_layer_surface->surface->mapped) {
       apply_exclusive (usable_area, state->anchor, state->exclusive_zone,
                        state->margin.top, state->margin.right,
                        state->margin.bottom, state->margin.left);
@@ -219,7 +220,7 @@ arrange_layer (PhocOutput                     *output,
     // same, because those with resized buffers will be handled separately.
 
     if (layer_surface->geo.x != old_geo.x || layer_surface->geo.y != old_geo.y)
-      update_cursors(layer_surface, seats);
+      update_cursors (layer_surface, seats);
   }
 }
 
@@ -237,8 +238,8 @@ phoc_layer_shell_find_osk (PhocOutput *output)
 {
   PhocLayerSurface *layer_surface;
 
-  wl_list_for_each(layer_surface, &output->layer_surfaces, link) {
-    if (strcmp(layer_surface->layer_surface->namespace, "osk") == 0)
+  wl_list_for_each (layer_surface, &output->layer_surfaces, link) {
+    if (strcmp (layer_surface->layer_surface->namespace, "osk") == 0)
       return layer_surface;
   }
 
@@ -249,9 +250,11 @@ phoc_layer_shell_find_osk (PhocOutput *output)
 void
 phoc_layer_shell_arrange (PhocOutput *output)
 {
-  struct wlr_box usable_area = { 0 };
   PhocServer *server = phoc_server_get_default ();
-  GSList *seats = phoc_input_get_seats (server->input);
+  PhocDesktop *desktop = phoc_server_get_desktop (server);
+  PhocInput *input = phoc_server_get_input (server);
+  struct wlr_box usable_area = { 0 };
+  GSList *seats = phoc_input_get_seats (input);
   enum zwlr_layer_shell_v1_layer layers[] = {
     ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY,
     ZWLR_LAYER_SHELL_V1_LAYER_TOP,
@@ -267,28 +270,23 @@ phoc_layer_shell_arrange (PhocOutput *output)
 
   wlr_output_effective_resolution (output->wlr_output, &usable_area.width, &usable_area.height);
   // Arrange exclusive surfaces from top->bottom
-  for (size_t i = 0; i < G_N_ELEMENTS(layers); ++i)
+  for (size_t i = 0; i < G_N_ELEMENTS (layers); ++i)
     arrange_layer (output, seats, layers[i], &usable_area, true);
   output->usable_area = usable_area;
 
-  PhocView *view;
-  wl_list_for_each (view, &output->desktop->views, link) {
-    if (view_is_maximized (view)) {
-      view_arrange_maximized (view, NULL);
-    } else if (view_is_tiled (view)) {
-      view_arrange_tiled (view, NULL);
-    } else if (output->desktop->maximize) {
-      view_center (view, NULL);
-    }
+  for (GList *l = phoc_desktop_get_views (desktop)->head; l; l = l->next) {
+    PhocView *view = PHOC_VIEW (l->data);
+
+    phoc_view_arrange (view, NULL, output->desktop->maximize);
   }
 
   // Arrange non-exlusive surfaces from top->bottom
-  for (size_t i = 0; i < G_N_ELEMENTS(layers); ++i)
+  for (size_t i = 0; i < G_N_ELEMENTS (layers); ++i)
     arrange_layer (output, seats, layers[i], &usable_area, false);
 
   phoc_output_update_shell_reveal (output);
 
-  if (G_UNLIKELY (server->debug_flags & PHOC_SERVER_DEBUG_FLAG_LAYER_SHELL)) {
+  if (G_UNLIKELY (phoc_server_check_debug_flags (server, PHOC_SERVER_DEBUG_FLAG_LAYER_SHELL))) {
     PhocLayerSurface *layer_surface;
     g_message ("Dumping layers:");
     wl_list_for_each (layer_surface, &output->layer_surfaces, link) {
@@ -305,10 +303,13 @@ phoc_layer_shell_arrange (PhocOutput *output)
   }
 }
 
+
 void
 phoc_layer_shell_update_focus (void)
 {
   PhocServer *server = phoc_server_get_default ();
+  PhocInput *input = phoc_server_get_input (server);
+  PhocDesktop *desktop = phoc_server_get_desktop (server);
   enum zwlr_layer_shell_v1_layer layers_above_shell[] = {
     ZWLR_LAYER_SHELL_V1_LAYER_OVERLAY,
     ZWLR_LAYER_SHELL_V1_LAYER_TOP,
@@ -317,8 +318,8 @@ phoc_layer_shell_update_focus (void)
   // Find topmost keyboard interactive layer, if such a layer exists
   // TODO: Make layer surface focus per-output based on cursor position
   PhocOutput *output;
-  wl_list_for_each (output, &server->desktop->outputs, link) {
-    for (size_t i = 0; i < G_N_ELEMENTS(layers_above_shell); ++i) {
+  wl_list_for_each (output, &desktop->outputs, link) {
+    for (size_t i = 0; i < G_N_ELEMENTS (layers_above_shell); ++i) {
       wl_list_for_each_reverse (layer_surface, &output->layer_surfaces, link) {
         if (layer_surface->layer != layers_above_shell[i])
           continue;
@@ -327,7 +328,7 @@ phoc_layer_shell_update_focus (void)
           continue;
 
         if (layer_surface->layer_surface->current.keyboard_interactive &&
-            layer_surface->layer_surface->mapped) {
+            layer_surface->layer_surface->surface->mapped) {
           topmost = layer_surface;
           break;
         }
@@ -344,7 +345,7 @@ phoc_layer_shell_update_focus (void)
           continue;
 
         if (layer_surface->layer_surface->current.keyboard_interactive &&
-            layer_surface->layer_surface->mapped) {
+            layer_surface->layer_surface->surface->mapped) {
           topmost = layer_surface;
           break;
         }
@@ -354,19 +355,21 @@ phoc_layer_shell_update_focus (void)
     }
   }
 
-  for (GSList *elem = phoc_input_get_seats (server->input); elem; elem = elem->next) {
+  for (GSList *elem = phoc_input_get_seats (input); elem; elem = elem->next) {
     PhocSeat *seat = PHOC_SEAT (elem->data);
 
     g_assert (PHOC_IS_SEAT (seat));
-    phoc_seat_set_focus_layer(seat, topmost ? topmost->layer_surface : NULL);
+    phoc_seat_set_focus_layer (seat, topmost ? topmost->layer_surface : NULL);
   }
 }
 
+
 static void
 handle_surface_commit (struct wl_listener *listener, void *data)
 {
   PhocServer *server = phoc_server_get_default ();
-  PhocLayerSurface *layer_surface = wl_container_of(listener, layer_surface, surface_commit);
+  PhocInput *input = phoc_server_get_input (server);
+  PhocLayerSurface *layer_surface = wl_container_of (listener, layer_surface, surface_commit);
   struct wlr_layer_surface_v1 *wlr_layer_surface = layer_surface->layer_surface;
   struct wlr_output *wlr_output = wlr_layer_surface->output;
   if (wlr_output != NULL) {
@@ -392,19 +395,25 @@ handle_surface_commit (struct wl_listener *listener, void *data)
     struct wlr_surface *surface = wlr_layer_surface->surface;
     if (surface->previous.width != surface->current.width ||
         surface->previous.height != surface->current.height) {
-      update_cursors(layer_surface, phoc_input_get_seats (server->input));
+      update_cursors (layer_surface, phoc_input_get_seats (input));
     }
 
     bool geo_changed =
-      memcmp(&old_geo, &layer_surface->geo, sizeof(struct wlr_box)) != 0;
+      memcmp (&old_geo, &layer_surface->geo, sizeof (struct wlr_box)) != 0;
     if (geo_changed || layer_changed) {
-      phoc_output_damage_whole_local_surface(output, wlr_layer_surface->surface,
-                                             old_geo.x, old_geo.y);
-      phoc_output_damage_whole_local_surface(output, wlr_layer_surface->surface,
-                                             layer_surface->geo.x, layer_surface->geo.y);
+      phoc_output_damage_whole_surface (output,
+                                        wlr_layer_surface->surface,
+                                        old_geo.x,
+                                        old_geo.y);
+      phoc_output_damage_whole_surface (output,
+                                        wlr_layer_surface->surface,
+                                        layer_surface->geo.x,
+                                        layer_surface->geo.y);
     } else {
-      phoc_output_damage_from_local_surface(output, wlr_layer_surface->surface,
-                                            layer_surface->geo.x, layer_surface->geo.y);
+      phoc_output_damage_from_surface (output,
+                                       wlr_layer_surface->surface,
+                                       layer_surface->geo.x,
+                                       layer_surface->geo.y);
     }
   }
 }
@@ -413,37 +422,40 @@ handle_surface_commit (struct wl_listener *listener, void *data)
 static void
 handle_destroy (struct wl_listener *listener, void *data)
 {
-  PhocLayerSurface *layer_surface = wl_container_of(listener, layer_surface, destroy);
+  PhocLayerSurface *layer_surface = wl_container_of (listener, layer_surface, destroy);
   g_object_unref (layer_surface);
 }
 
+
 static void
-subsurface_destroy (struct phoc_layer_subsurface *subsurface)
+phoc_layer_subsurface_destroy (PhocLayerSubsurface *subsurface)
 {
-  wl_list_remove(&subsurface->map.link);
-  wl_list_remove(&subsurface->unmap.link);
-  wl_list_remove(&subsurface->destroy.link);
-  wl_list_remove(&subsurface->commit.link);
-  wl_list_remove(&subsurface->link);
-  free(subsurface);
+  wl_list_remove (&subsurface->map.link);
+  wl_list_remove (&subsurface->unmap.link);
+  wl_list_remove (&subsurface->destroy.link);
+  wl_list_remove (&subsurface->commit.link);
+  wl_list_remove (&subsurface->link);
+  free (subsurface);
 }
 
-static struct phoc_layer_popup *popup_create(struct wlr_xdg_popup *wlr_popup);
-static struct phoc_layer_subsurface *layer_subsurface_create(struct wlr_subsurface *wlr_subsurface);
+
+static PhocLayerPopup *popup_create (struct wlr_xdg_popup *wlr_popup);
+static PhocLayerSubsurface *phoc_layer_subsurface_create (struct wlr_subsurface *wlr_subsurface);
 
 static PhocLayerSurface *
-popup_get_root_layer (struct phoc_layer_popup *popup)
+popup_get_root_layer (PhocLayerPopup *popup)
 {
-  while (popup->parent_type == LAYER_PARENT_POPUP) {
+  while (popup->parent_type == LAYER_PARENT_POPUP)
     popup = popup->parent_popup;
-  }
+
   return popup->parent_layer;
 }
 
+
 static void
-popup_unconstrain (struct phoc_layer_popup *popup)
+popup_unconstrain (PhocLayerPopup *popup)
 {
-  PhocLayerSurface *layer = popup_get_root_layer(popup);
+  PhocLayerSurface *layer = popup_get_root_layer (popup);
   struct wlr_xdg_popup *wlr_popup = popup->wlr_popup;
 
   PhocOutput *output = phoc_layer_surface_get_output (layer);
@@ -457,11 +469,12 @@ popup_unconstrain (struct phoc_layer_popup *popup)
     .height = output->usable_area.height,
   };
 
-  wlr_xdg_popup_unconstrain_from_box(wlr_popup, &output_toplevel_sx_box);
+  wlr_xdg_popup_unconstrain_from_box (wlr_popup, &output_toplevel_sx_box);
 }
 
+
 static void
-popup_damage (struct phoc_layer_popup *layer_popup, bool whole)
+popup_damage (PhocLayerPopup *layer_popup, bool whole)
 {
   struct wlr_xdg_popup *popup = layer_popup->wlr_popup;
   struct wlr_surface *surface = popup->base->surface;
@@ -479,127 +492,137 @@ popup_damage (struct phoc_layer_popup *layer_popup, bool whole)
   oy += layer->geo.y;
 
   PhocOutput *output = phoc_layer_surface_get_output (layer);
-  if (!output) {
+  if (!output)
     return;
-  }
 
-  if (whole) {
-    phoc_output_damage_whole_local_surface(output, surface, ox, oy);
-  } else {
-    phoc_output_damage_from_local_surface(output, surface, ox, oy);
-  }
+  if (whole)
+    phoc_output_damage_whole_surface (output, surface, ox, oy);
+  else
+    phoc_output_damage_from_surface (output, surface, ox, oy);
 }
 
+
 static void
 popup_new_popup (struct wl_listener *listener, void *data)
 {
-  struct phoc_layer_popup *popup =
-    wl_container_of(listener, popup, new_popup);
+  PhocLayerPopup *popup = wl_container_of (listener, popup, new_popup);
   struct wlr_xdg_popup *wlr_popup = data;
-  struct phoc_layer_popup *new_popup = popup_create(wlr_popup);
+  PhocLayerPopup *new_popup = popup_create (wlr_popup);
+
   new_popup->parent_type = LAYER_PARENT_POPUP;
   new_popup->parent_popup = popup;
-  popup_unconstrain(new_popup);
+  popup_unconstrain (new_popup);
 }
 
+
 static void
 popup_new_subsurface (struct wl_listener *listener, void *data)
 {
-  struct phoc_layer_popup *popup =
-    wl_container_of(listener, popup, new_subsurface);
+  PhocLayerPopup *popup = wl_container_of (listener, popup, new_subsurface);
   struct wlr_subsurface *wlr_subsurface = data;
-  struct phoc_layer_subsurface *subsurface = layer_subsurface_create(wlr_subsurface);
+  PhocLayerSubsurface *subsurface = phoc_layer_subsurface_create (wlr_subsurface);
+
   subsurface->parent_type = LAYER_PARENT_POPUP;
   subsurface->parent_popup = popup;
-  wl_list_insert(&popup->subsurfaces, &subsurface->link);
+  wl_list_insert (&popup->subsurfaces, &subsurface->link);
 }
 
+
 static void
 popup_handle_map (struct wl_listener *listener, void *data)
 {
-  PhocServer *server = phoc_server_get_default ();
-  struct phoc_layer_popup *popup = wl_container_of(listener, popup, map);
-  PhocLayerSurface *layer = popup_get_root_layer(popup);
+  PhocInput *input = phoc_server_get_input (phoc_server_get_default ());
+  PhocLayerPopup *popup = wl_container_of (listener, popup, map);
+  PhocLayerSurface *layer = popup_get_root_layer (popup);
   struct wlr_output *wlr_output = layer->layer_surface->output;
-  if (!wlr_output) {
+
+  if (!wlr_output)
     return;
-  }
 
   struct wlr_subsurface *child;
-  wl_list_for_each(child, &popup->wlr_popup->base->surface->current.subsurfaces_below, current.link) {
-    struct phoc_layer_subsurface *new_subsurface = layer_subsurface_create(child);
+  wl_list_for_each (child,
+                    &popup->wlr_popup->base->surface->current.subsurfaces_below,
+                    current.link) {
+    PhocLayerSubsurface *new_subsurface = phoc_layer_subsurface_create (child);
+
     new_subsurface->parent_type = LAYER_PARENT_POPUP;
     new_subsurface->parent_popup = popup;
-    wl_list_insert(&popup->subsurfaces, &new_subsurface->link);
+    wl_list_insert (&popup->subsurfaces, &new_subsurface->link);
   }
-  wl_list_for_each(child, &popup->wlr_popup->base->surface->current.subsurfaces_above, current.link) {
-    struct phoc_layer_subsurface *new_subsurface = layer_subsurface_create(child);
+  wl_list_for_each (child,
+                    &popup->wlr_popup->base->surface->current.subsurfaces_above,
+                    current.link) {
+    PhocLayerSubsurface *new_subsurface = phoc_layer_subsurface_create (child);
+
     new_subsurface->parent_type = LAYER_PARENT_POPUP;
     new_subsurface->parent_popup = popup;
-    wl_list_insert(&popup->subsurfaces, &new_subsurface->link);
+    wl_list_insert (&popup->subsurfaces, &new_subsurface->link);
   }
   popup->new_subsurface.notify = popup_new_subsurface;
-  wl_signal_add(&popup->wlr_popup->base->surface->events.new_subsurface, &popup->new_subsurface);
+  wl_signal_add (&popup->wlr_popup->base->surface->events.new_subsurface, &popup->new_subsurface);
 
-  wlr_surface_send_enter(popup->wlr_popup->base->surface, wlr_output);
-  popup_damage(popup, true);
-  phoc_input_update_cursor_focus(server->input);
+  phoc_utils_wlr_surface_enter_output (popup->wlr_popup->base->surface, wlr_output);
+  popup_damage (popup, true);
+  phoc_input_update_cursor_focus (input);
 }
 
-static void popup_handle_unmap (struct wl_listener *listener, void *data)
+static void
+popup_handle_unmap (struct wl_listener *listener, void *data)
 {
-  PhocServer *server = phoc_server_get_default ();
-  struct phoc_layer_popup *popup = wl_container_of(listener, popup, unmap);
-  struct phoc_layer_subsurface *child, *tmp;
-  wl_list_for_each_safe(child, tmp, &popup->subsurfaces, link) {
-    subsurface_destroy(child);
-  }
-  wl_list_remove(&popup->new_subsurface.link);
-  popup_damage(popup, true);
-  phoc_input_update_cursor_focus(server->input);
+  PhocInput *input = phoc_server_get_input (phoc_server_get_default ());
+  PhocLayerPopup *popup = wl_container_of (listener, popup, unmap);
+  PhocLayerSubsurface *child, *tmp;
+
+  wl_list_for_each_safe (child, tmp, &popup->subsurfaces, link)
+    phoc_layer_subsurface_destroy (child);
+
+  wl_list_remove (&popup->new_subsurface.link);
+  popup_damage (popup, true);
+  phoc_input_update_cursor_focus (input);
 }
 
 static void
 popup_handle_commit (struct wl_listener *listener, void *data)
 {
-  struct phoc_layer_popup *popup = wl_container_of(listener, popup, commit);
-  popup_damage(popup, false);
+  PhocLayerPopup *popup = wl_container_of (listener, popup, commit);
+
+  popup_damage (popup, false);
 }
 
 static void
 popup_handle_destroy (struct wl_listener *listener, void *data)
 {
-  struct phoc_layer_popup *popup = wl_container_of(listener, popup, destroy);
+  PhocLayerPopup *popup = wl_container_of (listener, popup, destroy);
 
-  wl_list_remove(&popup->map.link);
-  wl_list_remove(&popup->unmap.link);
-  wl_list_remove(&popup->destroy.link);
-  wl_list_remove(&popup->commit.link);
-  wl_list_remove(&popup->new_popup.link);
-  free(popup);
+  wl_list_remove (&popup->map.link);
+  wl_list_remove (&popup->unmap.link);
+  wl_list_remove (&popup->destroy.link);
+  wl_list_remove (&popup->commit.link);
+  wl_list_remove (&popup->new_popup.link);
+  free (popup);
 }
 
-static struct phoc_layer_popup *
+static PhocLayerPopup *
 popup_create (struct wlr_xdg_popup *wlr_popup)
 {
-  struct phoc_layer_popup *popup =
-    calloc(1, sizeof(struct phoc_layer_popup));
-  if (popup == NULL) {
+  PhocLayerPopup *popup = calloc (1, sizeof (PhocLayerPopup));
+
+  if (popup == NULL)
     return NULL;
-  }
+
   popup->wlr_popup = wlr_popup;
   popup->map.notify = popup_handle_map;
-  wl_signal_add(&wlr_popup->base->events.map, &popup->map);
+  wl_signal_add (&wlr_popup->base->surface->events.map, &popup->map);
   popup->unmap.notify = popup_handle_unmap;
-  wl_signal_add(&wlr_popup->base->events.unmap, &popup->unmap);
+  wl_signal_add (&wlr_popup->base->surface->events.unmap, &popup->unmap);
   popup->destroy.notify = popup_handle_destroy;
-  wl_signal_add(&wlr_popup->base->events.destroy, &popup->destroy);
+  wl_signal_add (&wlr_popup->base->events.destroy, &popup->destroy);
   popup->commit.notify = popup_handle_commit;
-  wl_signal_add(&wlr_popup->base->surface->events.commit, &popup->commit);
+  wl_signal_add (&wlr_popup->base->surface->events.commit, &popup->commit);
   popup->new_popup.notify = popup_new_popup;
-  wl_signal_add(&wlr_popup->base->events.new_popup, &popup->new_popup);
+  wl_signal_add (&wlr_popup->base->events.new_popup, &popup->new_popup);
 
-  wl_list_init(&popup->subsurfaces);
+  wl_list_init (&popup->subsurfaces);
 
   return popup;
 }
@@ -607,137 +630,147 @@ popup_create (struct wlr_xdg_popup *wlr_popup)
 static void
 handle_new_popup (struct wl_listener *listener, void *data)
 {
-  PhocLayerSurface *phoc_layer_surface =
-    wl_container_of(listener, phoc_layer_surface, new_popup);
+  PhocLayerSurface *layer_surface = wl_container_of (listener, layer_surface, new_popup);
   struct wlr_xdg_popup *wlr_popup = data;
-  struct phoc_layer_popup *popup = popup_create(wlr_popup);
+  PhocLayerPopup *popup = popup_create (wlr_popup);
+
   popup->parent_type = LAYER_PARENT_LAYER;
-  popup->parent_layer = phoc_layer_surface;
-  popup_unconstrain(popup);
+  popup->parent_layer = layer_surface;
+  popup_unconstrain (popup);
 }
 
 static PhocLayerSurface *
-subsurface_get_root_layer (struct phoc_layer_subsurface *subsurface)
+subsurface_get_root_layer (PhocLayerSubsurface *subsurface)
 {
-  while (subsurface->parent_type == LAYER_PARENT_SUBSURFACE) {
+  while (subsurface->parent_type == LAYER_PARENT_SUBSURFACE)
     subsurface = subsurface->parent_subsurface;
-  }
-  if (subsurface->parent_type == LAYER_PARENT_POPUP) {
-    return popup_get_root_layer(subsurface->parent_popup);
-  }
+
+  if (subsurface->parent_type == LAYER_PARENT_POPUP)
+    return popup_get_root_layer (subsurface->parent_popup);
+
   return subsurface->parent_layer;
 }
 
 static void
-subsurface_damage (struct phoc_layer_subsurface *subsurface, bool whole)
+subsurface_damage (PhocLayerSubsurface *subsurface, bool whole)
 {
-  PhocLayerSurface *layer = subsurface_get_root_layer(subsurface);
+  PhocLayerSurface *layer = subsurface_get_root_layer (subsurface);
   PhocOutput *output = phoc_layer_surface_get_output (layer);
-  if (!output) {
+  if (!output)
     return;
-  }
+
   int ox = subsurface->wlr_subsurface->current.x + layer->geo.x;
   int oy = subsurface->wlr_subsurface->current.y + layer->geo.y;
   if (whole) {
-    phoc_output_damage_whole_local_surface(output,
-                                           subsurface->wlr_subsurface->surface,
-                                           ox, oy);
+    phoc_output_damage_whole_surface (output,
+                                      subsurface->wlr_subsurface->surface,
+                                      ox, oy);
   } else {
-    phoc_output_damage_from_local_surface(output,
-                                          subsurface->wlr_subsurface->surface,
-                                          ox, oy);
+    phoc_output_damage_from_surface (output,
+                                     subsurface->wlr_subsurface->surface,
+                                     ox, oy);
   }
 }
 
-static void subsurface_new_subsurface (struct wl_listener *listener, void *data)
+static void
+subsurface_new_subsurface (struct wl_listener *listener, void *data)
 {
-  struct phoc_layer_subsurface *subsurface =
-    wl_container_of(listener, subsurface, new_subsurface);
+  PhocLayerSubsurface *subsurface = wl_container_of (listener, subsurface, new_subsurface);
   struct wlr_subsurface *wlr_subsurface = data;
+  PhocLayerSubsurface *new_subsurface = phoc_layer_subsurface_create (wlr_subsurface);
 
-  struct phoc_layer_subsurface *new_subsurface = layer_subsurface_create(wlr_subsurface);
   new_subsurface->parent_type = LAYER_PARENT_SUBSURFACE;
   new_subsurface->parent_subsurface = subsurface;
-  wl_list_insert(&subsurface->subsurfaces, &new_subsurface->link);
+  wl_list_insert (&subsurface->subsurfaces, &new_subsurface->link);
 }
 
 static void
 subsurface_handle_map (struct wl_listener *listener, void *data)
 {
-  PhocServer *server = phoc_server_get_default ();
-  struct phoc_layer_subsurface *subsurface = wl_container_of(listener, subsurface, map);
+  PhocInput *input = phoc_server_get_input (phoc_server_get_default ());
+  PhocLayerSubsurface *subsurface = wl_container_of (listener, subsurface, map);
 
   struct wlr_subsurface *child;
-  wl_list_for_each(child, &subsurface->wlr_subsurface->surface->current.subsurfaces_below, current.link) {
-    struct phoc_layer_subsurface *new_subsurface = layer_subsurface_create(child);
+  wl_list_for_each (child,
+                    &subsurface->wlr_subsurface->surface->current.subsurfaces_below,
+                    current.link) {
+    PhocLayerSubsurface *new_subsurface = phoc_layer_subsurface_create (child);
+
     new_subsurface->parent_type = LAYER_PARENT_SUBSURFACE;
     new_subsurface->parent_subsurface = subsurface;
-    wl_list_insert(&subsurface->subsurfaces, &new_subsurface->link);
+    wl_list_insert (&subsurface->subsurfaces, &new_subsurface->link);
   }
-  wl_list_for_each(child, &subsurface->wlr_subsurface->surface->current.subsurfaces_above, current.link) {
-    struct phoc_layer_subsurface *new_subsurface = layer_subsurface_create(child);
+  wl_list_for_each (child,
+                    &subsurface->wlr_subsurface->surface->current.subsurfaces_above,
+                    current.link) {
+    PhocLayerSubsurface *new_subsurface = phoc_layer_subsurface_create (child);
+
     new_subsurface->parent_type = LAYER_PARENT_SUBSURFACE;
     new_subsurface->parent_subsurface = subsurface;
-    wl_list_insert(&subsurface->subsurfaces, &new_subsurface->link);
+    wl_list_insert (&subsurface->subsurfaces, &new_subsurface->link);
   }
   subsurface->new_subsurface.notify = subsurface_new_subsurface;
-  wl_signal_add(&subsurface->wlr_subsurface->surface->events.new_subsurface, &subsurface->new_subsurface);
+  wl_signal_add (&subsurface->wlr_subsurface->surface->events.new_subsurface,
+                 &subsurface->new_subsurface);
 
-  wlr_surface_send_enter(subsurface->wlr_subsurface->surface, subsurface_get_root_layer(subsurface)->layer_surface->output);
-  subsurface_damage(subsurface, true);
-  phoc_input_update_cursor_focus(server->input);
+  PhocLayerSurface *layer_surface = subsurface_get_root_layer (subsurface);
+  phoc_utils_wlr_surface_enter_output (subsurface->wlr_subsurface->surface,
+                                       layer_surface->layer_surface->output);
+  subsurface_damage (subsurface, true);
+  phoc_input_update_cursor_focus (input);
 }
 
 static void
 subsurface_handle_unmap (struct wl_listener *listener, void *data)
 {
-  PhocServer *server = phoc_server_get_default ();
-  struct phoc_layer_subsurface *subsurface = wl_container_of(listener, subsurface, unmap);
-  struct phoc_layer_subsurface *child, *tmp;
-  wl_list_for_each_safe(child, tmp, &subsurface->subsurfaces, link) {
-    subsurface_destroy(child);
-  }
-  wl_list_remove(&subsurface->new_subsurface.link);
-  subsurface_damage(subsurface, true);
-  phoc_input_update_cursor_focus(server->input);
+  PhocInput *input = phoc_server_get_input (phoc_server_get_default ());
+  PhocLayerSubsurface *subsurface = wl_container_of (listener, subsurface, unmap);
+  PhocLayerSubsurface *child, *tmp;
+
+  wl_list_for_each_safe (child, tmp, &subsurface->subsurfaces, link)
+    phoc_layer_subsurface_destroy (child);
+
+  wl_list_remove (&subsurface->new_subsurface.link);
+  subsurface_damage (subsurface, true);
+  phoc_input_update_cursor_focus (input);
 }
 
 static void
 subsurface_handle_commit (struct wl_listener *listener, void *data)
 {
-  struct phoc_layer_subsurface *subsurface = wl_container_of(listener, subsurface, commit);
-  subsurface_damage(subsurface, false);
+  PhocLayerSubsurface *subsurface = wl_container_of (listener, subsurface, commit);
+  subsurface_damage (subsurface, false);
 }
 
 static void
 subsurface_handle_destroy (struct wl_listener *listener, void *data)
 {
-  struct phoc_layer_subsurface *subsurface = wl_container_of(listener, subsurface, destroy);
+  PhocLayerSubsurface *subsurface = wl_container_of (listener, subsurface, destroy);
 
-  subsurface_destroy(subsurface);
+  phoc_layer_subsurface_destroy (subsurface);
 }
 
-static struct phoc_layer_subsurface *
-layer_subsurface_create(struct wlr_subsurface *wlr_subsurface)
+static PhocLayerSubsurface *
+phoc_layer_subsurface_create (struct wlr_subsurface *wlr_subsurface)
 {
-  struct phoc_layer_subsurface *subsurface = calloc(1, sizeof(struct phoc_layer_subsurface));
+  PhocLayerSubsurface *subsurface = calloc (1, sizeof (PhocLayerSubsurface));
 
-  if (subsurface == NULL) {
+  if (subsurface == NULL)
     return NULL;
-  }
+
   subsurface->wlr_subsurface = wlr_subsurface;
 
   subsurface->map.notify = subsurface_handle_map;
-  wl_signal_add(&wlr_subsurface->events.map, &subsurface->map);
+  wl_signal_add (&wlr_subsurface->surface->events.map, &subsurface->map);
   subsurface->unmap.notify = subsurface_handle_unmap;
-  wl_signal_add(&wlr_subsurface->events.unmap, &subsurface->unmap);
+  wl_signal_add (&wlr_subsurface->surface->events.unmap, &subsurface->unmap);
   subsurface->destroy.notify = subsurface_handle_destroy;
-  wl_signal_add(&wlr_subsurface->events.destroy, &subsurface->destroy);
+  wl_signal_add (&wlr_subsurface->events.destroy, &subsurface->destroy);
   subsurface->commit.notify = subsurface_handle_commit;
-  wl_signal_add(&wlr_subsurface->surface->events.commit, &subsurface->commit);
+  wl_signal_add (&wlr_subsurface->surface->events.commit, &subsurface->commit);
 
-  wl_list_init(&subsurface->subsurfaces);
-  wl_list_init(&subsurface->link);
+  wl_list_init (&subsurface->subsurfaces);
+  wl_list_init (&subsurface->link);
 
   return subsurface;
 }
@@ -745,82 +778,90 @@ layer_subsurface_create(struct wlr_subsurface *wlr_subsurface)
 static void
 handle_new_subsurface (struct wl_listener *listener, void *data)
 {
-  PhocLayerSurface *phoc_layer_surface = wl_container_of(listener, phoc_layer_surface, new_subsurface);
+  PhocLayerSurface *layer_surface = wl_container_of (listener, layer_surface, new_subsurface);
   struct wlr_subsurface *wlr_subsurface = data;
 
-  struct phoc_layer_subsurface *subsurface = layer_subsurface_create(wlr_subsurface);
+  PhocLayerSubsurface *subsurface = phoc_layer_subsurface_create (wlr_subsurface);
   subsurface->parent_type = LAYER_PARENT_LAYER;
-  subsurface->parent_layer = phoc_layer_surface;
-  wl_list_insert(&phoc_layer_surface->subsurfaces, &subsurface->link);
+  subsurface->parent_layer = layer_surface;
+  wl_list_insert (&layer_surface->subsurfaces, &subsurface->link);
 }
 
+
 static void
 handle_map (struct wl_listener *listener, void *data)
 {
-  struct wlr_layer_surface_v1 *wlr_layer_surface = data;
-  PhocLayerSurface *layer_surface = PHOC_LAYER_SURFACE (wlr_layer_surface->data);
+  PhocLayerSurface *layer_surface = wl_container_of (listener, layer_surface, map);
+  struct wlr_layer_surface_v1 *wlr_layer_surface = layer_surface->layer_surface;
   PhocOutput *output = phoc_layer_surface_get_output (layer_surface);
 
-  if (!output) {
+  if (!output)
     return;
-  }
 
   layer_surface->mapped = true;
 
-  struct wlr_subsurface *subsurface;
-  wl_list_for_each(subsurface, &wlr_layer_surface->surface->current.subsurfaces_below, current.link) {
-    struct phoc_layer_subsurface *phoc_subsurface = layer_subsurface_create(subsurface);
-    phoc_subsurface->parent_type = LAYER_PARENT_LAYER;
-    phoc_subsurface->parent_layer = layer_surface;
-    wl_list_insert(&layer_surface->subsurfaces, &phoc_subsurface->link);
+  struct wlr_subsurface *wlr_subsurface;
+  wl_list_for_each (wlr_subsurface,
+                    &wlr_layer_surface->surface->current.subsurfaces_below,
+                    current.link) {
+    PhocLayerSubsurface *subsurface = phoc_layer_subsurface_create (wlr_subsurface);
+    subsurface->parent_type = LAYER_PARENT_LAYER;
+    subsurface->parent_layer = layer_surface;
+    wl_list_insert (&layer_surface->subsurfaces, &subsurface->link);
   }
-  wl_list_for_each(subsurface, &wlr_layer_surface->surface->current.subsurfaces_above, current.link) {
-    struct phoc_layer_subsurface *phoc_subsurface = layer_subsurface_create(subsurface);
-    phoc_subsurface->parent_type = LAYER_PARENT_LAYER;
-    phoc_subsurface->parent_layer = layer_surface;
-    wl_list_insert(&layer_surface->subsurfaces, &phoc_subsurface->link);
+  wl_list_for_each (wlr_subsurface,
+                    &wlr_layer_surface->surface->current.subsurfaces_above,
+                    current.link) {
+    PhocLayerSubsurface *subsurface = phoc_layer_subsurface_create (wlr_subsurface);
+    subsurface->parent_type = LAYER_PARENT_LAYER;
+    subsurface->parent_layer = layer_surface;
+    wl_list_insert (&layer_surface->subsurfaces, &subsurface->link);
   }
 
   layer_surface->new_subsurface.notify = handle_new_subsurface;
-  wl_signal_add(&wlr_layer_surface->surface->events.new_subsurface, &layer_surface->new_subsurface);
+  wl_signal_add (&wlr_layer_surface->surface->events.new_subsurface, &layer_surface->new_subsurface);
 
-  phoc_output_damage_whole_local_surface(output,
-                                         wlr_layer_surface->surface, layer_surface->geo.x,
-                                         layer_surface->geo.y);
-  wlr_surface_send_enter(wlr_layer_surface->surface, output->wlr_output);
+  phoc_output_damage_whole_surface (output,
+                                    wlr_layer_surface->surface,
+                                    layer_surface->geo.x,
+                                    layer_surface->geo.y);
+
+  phoc_utils_wlr_surface_enter_output (wlr_layer_surface->surface, output->wlr_output);
 
   phoc_layer_shell_arrange (output);
   phoc_layer_shell_update_focus ();
 }
 
+
 static void
 handle_unmap (struct wl_listener *listener, void *data)
 {
-  PhocServer *server = phoc_server_get_default ();
-  PhocLayerSurface *layer_surface = wl_container_of(listener, layer_surface, unmap);
+  PhocInput *input = phoc_server_get_input (phoc_server_get_default ());
+  PhocLayerSurface *layer_surface = wl_container_of (listener, layer_surface, unmap);
   PhocOutput *output = phoc_layer_surface_get_output (layer_surface);
 
   layer_surface->mapped = false;
 
-  struct phoc_layer_subsurface *subsurface, *tmp;
-  wl_list_for_each_safe(subsurface, tmp, &layer_surface->subsurfaces, link) {
-    subsurface_destroy(subsurface);
-  }
-  wl_list_remove(&layer_surface->new_subsurface.link);
+  PhocLayerSubsurface *subsurface, *tmp;
+  wl_list_for_each_safe (subsurface, tmp, &layer_surface->subsurfaces, link)
+    phoc_layer_subsurface_destroy (subsurface);
+
+  wl_list_remove (&layer_surface->new_subsurface.link);
 
   phoc_layer_surface_unmap (layer_surface);
-  phoc_input_update_cursor_focus(server->input);
+  phoc_input_update_cursor_focus (input);
 
   if (output)
     phoc_layer_shell_arrange (output);
   phoc_layer_shell_update_focus ();
 }
 
+
 void
-handle_layer_shell_surface (struct wl_listener *listener, void *data)
+phoc_handle_layer_shell_surface (struct wl_listener *listener, void *data)
 {
   struct wlr_layer_surface_v1 *wlr_layer_surface = data;
-  PhocDesktop *desktop = wl_container_of(listener, desktop, layer_shell_surface);
+  PhocDesktop *desktop = wl_container_of (listener, desktop, layer_shell_surface);
 
   g_debug ("new layer surface: namespace %s layer %d anchor %d "
            "size %dx%d margin %d,%d,%d,%d",
@@ -836,42 +877,38 @@ handle_layer_shell_surface (struct wl_listener *listener, void *data)
   if (!wlr_layer_surface->output) {
     PhocSeat *seat = phoc_server_get_last_active_seat (phoc_server_get_default ());
     g_assert (PHOC_IS_SEAT (seat)); // Technically speaking we should handle this case
-    PhocCursor *cursor = phoc_seat_get_cursor(seat);
-    struct wlr_output *output =
-      wlr_output_layout_output_at(desktop->layout,
-                                  cursor->cursor->x,
-                                  cursor->cursor->y);
+    PhocCursor *cursor = phoc_seat_get_cursor (seat);
+    struct wlr_output *output = wlr_output_layout_output_at (desktop->layout,
+                                                             cursor->cursor->x,
+                                                             cursor->cursor->y);
     if (!output) {
-      g_critical ("Couldn't find output at (%.0f,%.0f)",
-                  cursor->cursor->x,
-                  cursor->cursor->y);
-      output = wlr_output_layout_get_center_output(desktop->layout);
+      g_critical ("Couldn't find output at (%.0f,%.0f)", cursor->cursor->x, cursor->cursor->y);
+      output = wlr_output_layout_get_center_output (desktop->layout);
     }
     if (output) {
       wlr_layer_surface->output = output;
     } else {
-      wlr_layer_surface_v1_destroy(wlr_layer_surface);
+      wlr_layer_surface_v1_destroy (wlr_layer_surface);
       return;
     }
   }
 
-
   PhocLayerSurface *layer_surface = phoc_layer_surface_new (wlr_layer_surface);
 
   layer_surface->surface_commit.notify = handle_surface_commit;
-  wl_signal_add(&wlr_layer_surface->surface->events.commit, &layer_surface->surface_commit);
+  wl_signal_add (&wlr_layer_surface->surface->events.commit, &layer_surface->surface_commit);
 
   layer_surface->destroy.notify = handle_destroy;
-  wl_signal_add(&wlr_layer_surface->events.destroy, &layer_surface->destroy);
+  wl_signal_add (&wlr_layer_surface->events.destroy, &layer_surface->destroy);
   layer_surface->map.notify = handle_map;
-  wl_signal_add(&wlr_layer_surface->events.map, &layer_surface->map);
+  wl_signal_add (&wlr_layer_surface->surface->events.map, &layer_surface->map);
   layer_surface->unmap.notify = handle_unmap;
-  wl_signal_add(&wlr_layer_surface->events.unmap, &layer_surface->unmap);
+  wl_signal_add (&wlr_layer_surface->surface->events.unmap, &layer_surface->unmap);
   layer_surface->new_popup.notify = handle_new_popup;
-  wl_signal_add(&wlr_layer_surface->events.new_popup, &layer_surface->new_popup);
+  wl_signal_add (&wlr_layer_surface->events.new_popup, &layer_surface->new_popup);
 
   PhocOutput *output = wlr_layer_surface->output->data;
-  wl_list_insert(&output->layer_surfaces, &layer_surface->link);
+  wl_list_insert (&output->layer_surfaces, &layer_surface->link);
 
   // Temporarily set the layer's current state to pending
   // So that we can easily arrange it
@@ -899,9 +936,9 @@ handle_layer_shell_surface (struct wl_listener *listener, void *data)
 void
 phoc_layer_shell_update_osk (PhocOutput *output, gboolean arrange)
 {
+  PhocInput *input = phoc_server_get_input (phoc_server_get_default ());
   PhocLayerSurface *osk;
-  PhocServer *server = phoc_server_get_default ();
-  GSList *seats = phoc_input_get_seats (server->input);
+  GSList *seats = phoc_input_get_seats (input);
   gboolean force_overlay = FALSE;
 
   g_assert (PHOC_IS_OUTPUT (output));
