@@ -6,6 +6,7 @@
 
 #pragma once
 
+#include "drag-icon.h"
 #include "input.h"
 #include "layers.h"
 #include "text_input.h"
@@ -23,7 +24,6 @@ G_BEGIN_DECLS
 G_DECLARE_FINAL_TYPE (PhocSeat, phoc_seat, PHOC, SEAT, GObject)
 
 typedef struct _PhocCursor PhocCursor;
-typedef struct _PhocDragIcon PhocDragIcon;
 typedef struct _PhocTablet PhocTablet;
 
 /**
@@ -69,6 +69,7 @@ typedef struct _PhocSeat {
   struct wl_listener              destroy;
 } PhocSeat;
 
+
 typedef struct _PhocSeatView {
   PhocSeat          *seat;
   PhocView          *view;
@@ -80,17 +81,6 @@ typedef struct _PhocSeatView {
   struct wl_list     link;   // PhocSeat::views
 } PhocSeatView;
 
-struct _PhocDragIcon {
-  PhocSeat             *seat;
-  struct wlr_drag_icon *wlr_drag_icon;
-
-  double                x, y;
-
-  struct wl_listener    surface_commit;
-  struct wl_listener    map;
-  struct wl_listener    unmap;
-  struct wl_listener    destroy;
-};
 
 typedef struct _PhocTabletPad {
   struct wl_list                   link;
@@ -109,6 +99,7 @@ typedef struct _PhocTabletPad {
   struct wl_listener               tablet_destroy;
 } PhocTabletPad;
 
+
 typedef struct _PhocTabletTool {
   struct wl_list                    link;
   struct wl_list                    tool_link;
@@ -124,6 +115,7 @@ typedef struct _PhocTabletTool {
   struct wl_listener                tablet_destroy;
 } PhocTabletTool;
 
+
 typedef struct PhocPointerConstraint {
   struct wlr_pointer_constraint_v1 *constraint;
 
@@ -142,7 +134,7 @@ PhocCursor        *phoc_seat_get_cursor (PhocSeat *self);
 
 void               phoc_seat_configure_xcursor (PhocSeat *seat);
 
-bool               phoc_seat_has_meta_pressed (PhocSeat *seat);
+bool               phoc_seat_grab_meta_press (PhocSeat *seat);
 
 PhocView          *phoc_seat_get_focus_view  (PhocSeat *seat);
 void               phoc_seat_set_focus_view  (PhocSeat *seat, PhocView *view);
@@ -160,10 +152,6 @@ void               phoc_seat_end_compositor_grab (PhocSeat *seat);
 
 PhocSeatView      *phoc_seat_view_from_view (PhocSeat *seat, PhocView *view);
 
-void               phoc_drag_icon_update_position (PhocDragIcon *icon);
-
-void               phoc_drag_icon_damage_whole (PhocDragIcon *icon);
-
 void               phoc_seat_set_exclusive_client (PhocSeat         *seat,
                                                    struct wl_client *client);
 
@@ -175,4 +163,5 @@ void               phoc_seat_maybe_set_cursor (PhocSeat *self, const char *name)
 gboolean           phoc_seat_has_touch    (PhocSeat *self);
 gboolean           phoc_seat_has_pointer  (PhocSeat *self);
 gboolean           phoc_seat_has_keyboard (PhocSeat *self);
+gboolean           phoc_seat_has_hw_keyboard (PhocSeat *self);
 gboolean           phoc_seat_has_switch   (PhocSeat *self, enum wlr_switch_type type);
