@@ -2,16 +2,11 @@
 
 #include "phoc-config.h"
 
-#ifndef _POSIX_C_SOURCE
-#define _POSIX_C_SOURCE 200809L
-#endif
-#include <limits.h>
-#include <stdlib.h>
-#include <string.h>
+#include <stdio.h>
 #include <strings.h>
 #include <sys/param.h>
-#include <unistd.h>
 
+#include "phoc-enums.h"
 #include "settings.h"
 #include "utils.h"
 
@@ -63,6 +58,24 @@ parse_modeline (const char *s, drmModeModeInfo *mode)
 }
 
 
+static
+PhocOutputScaleFilter
+parse_scale_filter (const char *value)
+{
+  GEnumValue *ev;
+  g_autoptr (GEnumClass) eclass = NULL;
+
+  eclass = G_ENUM_CLASS (g_type_class_ref (phoc_output_scale_filter_get_type ()));
+  ev = g_enum_get_value_by_nick (eclass, value);
+  if (!ev) {
+    g_critical ("Got invalid output scale-filter value: %s", value);
+    return PHOC_OUTPUT_SCALE_FILTER_AUTO;
+  }
+
+  return ev->value;
+}
+
+
 static const char *output_prefix = "output:";
 
 static PhocOutputConfig *
@@ -75,6 +88,9 @@ phoc_output_config_new (const char *name)
   oc->transform = WL_OUTPUT_TRANSFORM_NORMAL;
   oc->scale = 0;
   oc->enable = true;
+  oc->x = -1;
+  oc->y = -1;
+  oc->scale_filter = PHOC_OUTPUT_SCALE_FILTER_AUTO;
 
   return oc;
 }
@@ -188,6 +204,8 @@ config_ini_handler (PhocConfig *config, const char *section, const char *name, c
         oc->modes = g_slist_prepend (oc->modes, g_steal_pointer (&mode));
       else
         g_critical ("Invalid modeline: %s", value);
+    } else if (strcmp (name, "scale-filter") == 0) {
+      oc->scale_filter = parse_scale_filter (value);
     }
   } else {
     g_critical ("Found unknown config section: %s", section);
@@ -323,6 +341,7 @@ phoc_config_destroy (PhocConfig *config)
   g_free (config);
 }
 
+
 static gboolean
 output_is_match (PhocOutputConfig *oc, PhocOutput *output)
 {
@@ -331,21 +350,32 @@ output_is_match (PhocOutputConfig *oc, PhocOutput *output)
   if (g_strcmp0 (oc->name, phoc_output_get_name (output)) == 0)
     return TRUE;
 
-  /* "vendor make model" match */
-  vmm = g_strsplit (oc->name, " ", 4);
+  /* "make%model%serial" match */
+  vmm = g_strsplit (oc->name, "%", 4);
   if (g_strv_length (vmm) != 3)
     return FALSE;
 
-  return phoc_output_is_match (output, vmm[0], vmm[1], vmm[2]);
+  if (!(g_strcmp0 (vmm[0], "*") == 0 || g_strcmp0 (vmm[0], output->wlr_output->make) == 0))
+    return FALSE;
+
+  if (!(g_strcmp0 (vmm[1], "*") == 0 || g_strcmp0 (vmm[1], output->wlr_output->model) == 0))
+    return FALSE;
+
+  if (!(g_strcmp0 (vmm[2], "*") == 0 || g_strcmp0 (vmm[2], output->wlr_output->serial) == 0))
+    return FALSE;
+
+  return TRUE;
 }
 
 /**
  * phoc_config_get_output:
  * config: The #PhocConfig
- * output: The wlr output to get the configuration for
+ * output: The output to get the configuration for
  *
- * Get configuration for the output. If the output is not configured, returns
- * NULL.
+ * Get intended configuration for the given output.
+ *
+ * Returns: The intended output configuration or %NULL or not
+ *     configuration is found.
  */
 PhocOutputConfig *
 phoc_config_get_output (PhocConfig *config, PhocOutput *output)
